# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'CGNS/NAV/T/Q7FileWindow.ui'
#
# Created by: PyQt5 UI code generator 5.12.3
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Q7FileWindow(object):
    def setupUi(self, Q7FileWindow):
        Q7FileWindow.setObjectName("Q7FileWindow")
        Q7FileWindow.setWindowModality(QtCore.Qt.ApplicationModal)
        Q7FileWindow.resize(728, 450)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Q7FileWindow.sizePolicy().hasHeightForWidth())
        Q7FileWindow.setSizePolicy(sizePolicy)
        Q7FileWindow.setMinimumSize(QtCore.QSize(728, 450))
        Q7FileWindow.setMaximumSize(QtCore.QSize(800, 600))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/images/icons/cgSpy.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Q7FileWindow.setWindowIcon(icon)
        self.verticalLayout = QtWidgets.QVBoxLayout(Q7FileWindow)
        self.verticalLayout.setContentsMargins(4, 0, 4, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(-1, -1, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.tabs = QtWidgets.QTabWidget(Q7FileWindow)
        self.tabs.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabs.sizePolicy().hasHeightForWidth())
        self.tabs.setSizePolicy(sizePolicy)
        self.tabs.setMinimumSize(QtCore.QSize(0, 0))
        self.tabs.setMaximumSize(QtCore.QSize(800, 440))
        self.tabs.setObjectName("tabs")
        self.Selection = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.Selection.sizePolicy().hasHeightForWidth())
        self.Selection.setSizePolicy(sizePolicy)
        self.Selection.setMinimumSize(QtCore.QSize(0, 0))
        self.Selection.setMaximumSize(QtCore.QSize(700, 400))
        self.Selection.setObjectName("Selection")
        self.selectionVL = QtWidgets.QVBoxLayout(self.Selection)
        self.selectionVL.setContentsMargins(8, 8, 8, 8)
        self.selectionVL.setSpacing(6)
        self.selectionVL.setObjectName("selectionVL")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setSpacing(3)
        self.gridLayout.setObjectName("gridLayout")
        self.fileentries = QtWidgets.QComboBox(self.Selection)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.fileentries.sizePolicy().hasHeightForWidth())
        self.fileentries.setSizePolicy(sizePolicy)
        self.fileentries.setEditable(True)
        self.fileentries.setObjectName("fileentries")
        self.gridLayout.addWidget(self.fileentries, 5, 2, 1, 2)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.cNoLargeData = QtWidgets.QCheckBox(self.Selection)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.cNoLargeData.setFont(font)
        self.cNoLargeData.setObjectName("cNoLargeData")
        self.horizontalLayout_4.addWidget(self.cNoLargeData)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem)
        self.gridLayout.addLayout(self.horizontalLayout_4, 1, 2, 1, 3)
        self.direntries = QtWidgets.QComboBox(self.Selection)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.direntries.sizePolicy().hasHeightForWidth())
        self.direntries.setSizePolicy(sizePolicy)
        self.direntries.setEditable(True)
        self.direntries.setObjectName("direntries")
        self.gridLayout.addWidget(self.direntries, 3, 2, 1, 3)
        self.cShowDirs = QtWidgets.QCheckBox(self.Selection)
        self.cShowDirs.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.cShowDirs.setChecked(True)
        self.cShowDirs.setObjectName("cShowDirs")
        self.gridLayout.addWidget(self.cShowDirs, 5, 4, 1, 1)
        self.bCurrent = QtWidgets.QPushButton(self.Selection)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bCurrent.sizePolicy().hasHeightForWidth())
        self.bCurrent.setSizePolicy(sizePolicy)
        self.bCurrent.setMinimumSize(QtCore.QSize(25, 25))
        self.bCurrent.setMaximumSize(QtCore.QSize(25, 25))
        self.bCurrent.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.bCurrent.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/images/icons/local-dir.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bCurrent.setIcon(icon1)
        self.bCurrent.setObjectName("bCurrent")
        self.gridLayout.addWidget(self.bCurrent, 3, 1, 1, 1)
        self.bBack = QtWidgets.QPushButton(self.Selection)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bBack.sizePolicy().hasHeightForWidth())
        self.bBack.setSizePolicy(sizePolicy)
        self.bBack.setMinimumSize(QtCore.QSize(25, 25))
        self.bBack.setMaximumSize(QtCore.QSize(25, 25))
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.bBack.setFont(font)
        self.bBack.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/images/icons/parent-dir.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bBack.setIcon(icon2)
        self.bBack.setIconSize(QtCore.QSize(16, 16))
        self.bBack.setObjectName("bBack")
        self.gridLayout.addWidget(self.bBack, 3, 0, 1, 1)
        self.selectionVL.addLayout(self.gridLayout)
        self.treeview = QtWidgets.QTreeView(self.Selection)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.treeview.sizePolicy().hasHeightForWidth())
        self.treeview.setSizePolicy(sizePolicy)
        self.treeview.setMinimumSize(QtCore.QSize(650, 235))
        self.treeview.setMaximumSize(QtCore.QSize(720, 450))
        self.treeview.setEditTriggers(QtWidgets.QAbstractItemView.EditKeyPressed|QtWidgets.QAbstractItemView.SelectedClicked)
        self.treeview.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.treeview.setUniformRowHeights(True)
        self.treeview.setSortingEnabled(True)
        self.treeview.setAllColumnsShowFocus(True)
        self.treeview.setObjectName("treeview")
        self.selectionVL.addWidget(self.treeview)
        self.bottomAction = QtWidgets.QHBoxLayout()
        self.bottomAction.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.bottomAction.setContentsMargins(-1, -1, 0, -1)
        self.bottomAction.setSpacing(6)
        self.bottomAction.setObjectName("bottomAction")
        self.bInfo = QtWidgets.QPushButton(self.Selection)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bInfo.sizePolicy().hasHeightForWidth())
        self.bInfo.setSizePolicy(sizePolicy)
        self.bInfo.setMinimumSize(QtCore.QSize(25, 25))
        self.bInfo.setMaximumSize(QtCore.QSize(25, 25))
        self.bInfo.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/images/icons/help-view.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bInfo.setIcon(icon3)
        self.bInfo.setObjectName("bInfo")
        self.bottomAction.addWidget(self.bInfo)
        spacerItem1 = QtWidgets.QSpacerItem(40, 10, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.bottomAction.addItem(spacerItem1)
        self.bAction = QtWidgets.QPushButton(self.Selection)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bAction.sizePolicy().hasHeightForWidth())
        self.bAction.setSizePolicy(sizePolicy)
        self.bAction.setMaximumSize(QtCore.QSize(70, 16777215))
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.bAction.setFont(font)
        self.bAction.setObjectName("bAction")
        self.bottomAction.addWidget(self.bAction)
        self.bClose = QtWidgets.QPushButton(self.Selection)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.bClose.sizePolicy().hasHeightForWidth())
        self.bClose.setSizePolicy(sizePolicy)
        self.bClose.setMinimumSize(QtCore.QSize(0, 0))
        self.bClose.setMaximumSize(QtCore.QSize(70, 16777215))
        self.bClose.setObjectName("bClose")
        self.bottomAction.addWidget(self.bClose)
        self.selectionVL.addLayout(self.bottomAction)
        self.tabs.addTab(self.Selection, "")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.load_save_Lay = QtWidgets.QGridLayout(self.tab)
        self.load_save_Lay.setContentsMargins(8, 8, 8, 8)
        self.load_save_Lay.setSpacing(2)
        self.load_save_Lay.setObjectName("load_save_Lay")
        self.groupBox_4 = QtWidgets.QGroupBox(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_4.sizePolicy().hasHeightForWidth())
        self.groupBox_4.setSizePolicy(sizePolicy)
        self.groupBox_4.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.groupBox_4.setObjectName("groupBox_4")
        self.saveVL = QtWidgets.QVBoxLayout(self.groupBox_4)
        self.saveVL.setContentsMargins(8, 8, 8, 8)
        self.saveVL.setSpacing(2)
        self.saveVL.setObjectName("saveVL")
        self.cOverwrite = QtWidgets.QCheckBox(self.groupBox_4)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.cOverwrite.setFont(font)
        self.cOverwrite.setObjectName("cOverwrite")
        self.saveVL.addWidget(self.cOverwrite)
        self.cDeleteMissing = QtWidgets.QCheckBox(self.groupBox_4)
        self.cDeleteMissing.setObjectName("cDeleteMissing")
        self.saveVL.addWidget(self.cDeleteMissing)
        self.cSkipEmpty = QtWidgets.QCheckBox(self.groupBox_4)
        self.cSkipEmpty.setObjectName("cSkipEmpty")
        self.saveVL.addWidget(self.cSkipEmpty)
        self.cSaveWithoutLinks = QtWidgets.QCheckBox(self.groupBox_4)
        self.cSaveWithoutLinks.setObjectName("cSaveWithoutLinks")
        self.saveVL.addWidget(self.cSaveWithoutLinks)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.saveVL.addItem(spacerItem2)
        self.load_save_Lay.addWidget(self.groupBox_4, 0, 1, 1, 1)
        self.groupBox_5 = QtWidgets.QGroupBox(self.tab)
        self.groupBox_5.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.groupBox_5.setObjectName("groupBox_5")
        self.CHLoneOptionGrid = QtWidgets.QGridLayout(self.groupBox_5)
        self.CHLoneOptionGrid.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.CHLoneOptionGrid.setContentsMargins(8, 8, 8, 8)
        self.CHLoneOptionGrid.setSpacing(2)
        self.CHLoneOptionGrid.setObjectName("CHLoneOptionGrid")
        self.checkBox = QtWidgets.QCheckBox(self.groupBox_5)
        self.checkBox.setObjectName("checkBox")
        self.CHLoneOptionGrid.addWidget(self.checkBox, 0, 0, 1, 1)
        self.checkBox_2 = QtWidgets.QCheckBox(self.groupBox_5)
        self.checkBox_2.setObjectName("checkBox_2")
        self.CHLoneOptionGrid.addWidget(self.checkBox_2, 2, 0, 1, 2)
        self.line = QtWidgets.QFrame(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line.sizePolicy().hasHeightForWidth())
        self.line.setSizePolicy(sizePolicy)
        self.line.setFrameShape(QtWidgets.QFrame.VLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.CHLoneOptionGrid.addWidget(self.line, 0, 2, 7, 1)
        spacerItem3 = QtWidgets.QSpacerItem(298, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.CHLoneOptionGrid.addItem(spacerItem3, 0, 3, 1, 1)
        self.checkBox_3 = QtWidgets.QCheckBox(self.groupBox_5)
        self.checkBox_3.setObjectName("checkBox_3")
        self.CHLoneOptionGrid.addWidget(self.checkBox_3, 3, 0, 1, 1)
        self.lineEdit_2 = QtWidgets.QLineEdit(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit_2.sizePolicy().hasHeightForWidth())
        self.lineEdit_2.setSizePolicy(sizePolicy)
        self.lineEdit_2.setMinimumSize(QtCore.QSize(0, 0))
        self.lineEdit_2.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.lineEdit_2.setAcceptDrops(False)
        self.lineEdit_2.setMaxLength(32760)
        self.lineEdit_2.setFrame(True)
        self.lineEdit_2.setObjectName("lineEdit_2")
        self.CHLoneOptionGrid.addWidget(self.lineEdit_2, 4, 0, 1, 2)
        spacerItem4 = QtWidgets.QSpacerItem(200, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.CHLoneOptionGrid.addItem(spacerItem4, 5, 0, 1, 1)
        self.load_save_Lay.addWidget(self.groupBox_5, 1, 0, 1, 2)
        self.groupBox_3 = QtWidgets.QGroupBox(self.tab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_3.sizePolicy().hasHeightForWidth())
        self.groupBox_3.setSizePolicy(sizePolicy)
        self.groupBox_3.setMinimumSize(QtCore.QSize(0, 0))
        self.groupBox_3.setMaximumSize(QtCore.QSize(364, 16777215))
        self.groupBox_3.setObjectName("groupBox_3")
        self.LoadVL = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.LoadVL.setContentsMargins(8, 8, 8, 8)
        self.LoadVL.setSpacing(2)
        self.LoadVL.setObjectName("LoadVL")
        self.cNoLargeData_2 = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setBold(False)
        font.setWeight(50)
        self.cNoLargeData_2.setFont(font)
        self.cNoLargeData_2.setObjectName("cNoLargeData_2")
        self.LoadVL.addWidget(self.cNoLargeData_2)
        self.cFollowLinks = QtWidgets.QCheckBox(self.groupBox_3)
        self.cFollowLinks.setObjectName("cFollowLinks")
        self.LoadVL.addWidget(self.cFollowLinks)
        self.cReadOnly = QtWidgets.QCheckBox(self.groupBox_3)
        self.cReadOnly.setEnabled(True)
        self.cReadOnly.setObjectName("cReadOnly")
        self.LoadVL.addWidget(self.cReadOnly)
        self.cLoadSubPath = QtWidgets.QCheckBox(self.groupBox_3)
        self.cLoadSubPath.setObjectName("cLoadSubPath")
        self.LoadVL.addWidget(self.cLoadSubPath)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem5 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem5)
        self.lineEdit = QtWidgets.QLineEdit(self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEdit.sizePolicy().hasHeightForWidth())
        self.lineEdit.setSizePolicy(sizePolicy)
        self.lineEdit.setObjectName("lineEdit")
        self.horizontalLayout_2.addWidget(self.lineEdit)
        self.LoadVL.addLayout(self.horizontalLayout_2)
        self.LimitDepthHLay = QtWidgets.QHBoxLayout()
        self.LimitDepthHLay.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.LimitDepthHLay.setObjectName("LimitDepthHLay")
        self.cLimitDepth = QtWidgets.QCheckBox(self.groupBox_3)
        self.cLimitDepth.setObjectName("cLimitDepth")
        self.LimitDepthHLay.addWidget(self.cLimitDepth)
        self.spinBox = QtWidgets.QSpinBox(self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinBox.sizePolicy().hasHeightForWidth())
        self.spinBox.setSizePolicy(sizePolicy)
        self.spinBox.setMinimumSize(QtCore.QSize(60, 0))
        self.spinBox.setObjectName("spinBox")
        self.LimitDepthHLay.addWidget(self.spinBox)
        spacerItem6 = QtWidgets.QSpacerItem(20, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.LimitDepthHLay.addItem(spacerItem6)
        self.LoadVL.addLayout(self.LimitDepthHLay)
        spacerItem7 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.LoadVL.addItem(spacerItem7)
        self.load_save_Lay.addWidget(self.groupBox_3, 0, 0, 1, 1)
        self.tabs.addTab(self.tab, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tab_2.sizePolicy().hasHeightForWidth())
        self.tab_2.setSizePolicy(sizePolicy)
        self.tab_2.setMinimumSize(QtCore.QSize(695, 390))
        self.tab_2.setMaximumSize(QtCore.QSize(720, 420))
        self.tab_2.setObjectName("tab_2")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.tab_2)
        self.verticalLayout_3.setContentsMargins(6, 6, 6, 6)
        self.verticalLayout_3.setSpacing(2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.topHL = QtWidgets.QHBoxLayout()
        self.topHL.setSpacing(0)
        self.topHL.setObjectName("topHL")
        self.cActivate = QtWidgets.QCheckBox(self.tab_2)
        self.cActivate.setEnabled(False)
        self.cActivate.setMaximumSize(QtCore.QSize(260, 16777215))
        self.cActivate.setChecked(True)
        self.cActivate.setObjectName("cActivate")
        self.topHL.addWidget(self.cActivate)
        self.cAutoDir = QtWidgets.QCheckBox(self.tab_2)
        self.cAutoDir.setObjectName("cAutoDir")
        self.topHL.addWidget(self.cAutoDir)
        self.verticalLayout_3.addLayout(self.topHL)
        self.downHL = QtWidgets.QHBoxLayout()
        self.downHL.setObjectName("downHL")
        self.groupBox_2 = QtWidgets.QGroupBox(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setMaximumSize(QtCore.QSize(190, 16777215))
        self.groupBox_2.setObjectName("groupBox_2")
        self.showVL = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.showVL.setContentsMargins(4, 4, 4, 4)
        self.showVL.setSpacing(4)
        self.showVL.setObjectName("showVL")
        self.cShowAll = QtWidgets.QCheckBox(self.groupBox_2)
        self.cShowAll.setObjectName("cShowAll")
        self.showVL.addWidget(self.cShowAll)
        self.__O_filterhdffiles = QtWidgets.QCheckBox(self.groupBox_2)
        self.__O_filterhdffiles.setChecked(True)
        self.__O_filterhdffiles.setObjectName("__O_filterhdffiles")
        self.showVL.addWidget(self.__O_filterhdffiles)
        self.__O_filtercgnsfiles = QtWidgets.QCheckBox(self.groupBox_2)
        self.__O_filtercgnsfiles.setChecked(True)
        self.__O_filtercgnsfiles.setObjectName("__O_filtercgnsfiles")
        self.showVL.addWidget(self.__O_filtercgnsfiles)
        spacerItem8 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.showVL.addItem(spacerItem8)
        self.cShowOwnExt = QtWidgets.QCheckBox(self.groupBox_2)
        self.cShowOwnExt.setObjectName("cShowOwnExt")
        self.showVL.addWidget(self.cShowOwnExt)
        self.lOwnExt = QtWidgets.QListWidget(self.groupBox_2)
        self.lOwnExt.setObjectName("lOwnExt")
        self.showVL.addWidget(self.lOwnExt)
        self.downHL.addWidget(self.groupBox_2)
        self.groupBox = QtWidgets.QGroupBox(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_4.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout_4.setSpacing(1)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.rClearNotFound = QtWidgets.QRadioButton(self.groupBox)
        self.rClearNotFound.setChecked(True)
        self.rClearNotFound.setObjectName("rClearNotFound")
        self.horizontalLayout_3.addWidget(self.rClearNotFound)
        spacerItem9 = QtWidgets.QSpacerItem(10, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem9)
        self.bInfo2 = QtWidgets.QPushButton(self.groupBox)
        self.bInfo2.setMinimumSize(QtCore.QSize(25, 25))
        self.bInfo2.setMaximumSize(QtCore.QSize(25, 25))
        self.bInfo2.setText("")
        self.bInfo2.setIcon(icon3)
        self.bInfo2.setObjectName("bInfo2")
        self.horizontalLayout_3.addWidget(self.bInfo2)
        self.bClearHistory = QtWidgets.QPushButton(self.groupBox)
        self.bClearHistory.setMaximumSize(QtCore.QSize(70, 25))
        self.bClearHistory.setObjectName("bClearHistory")
        self.horizontalLayout_3.addWidget(self.bClearHistory)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.rClearNoHDF = QtWidgets.QRadioButton(self.groupBox)
        self.rClearNoHDF.setObjectName("rClearNoHDF")
        self.verticalLayout_4.addWidget(self.rClearNoHDF)
        self.rClearSelectedDirs = QtWidgets.QRadioButton(self.groupBox)
        self.rClearSelectedDirs.setObjectName("rClearSelectedDirs")
        self.verticalLayout_4.addWidget(self.rClearSelectedDirs)
        self.rClearSelectedFiles = QtWidgets.QRadioButton(self.groupBox)
        self.rClearSelectedFiles.setObjectName("rClearSelectedFiles")
        self.verticalLayout_4.addWidget(self.rClearSelectedFiles)
        self.rClearAllDirs = QtWidgets.QRadioButton(self.groupBox)
        self.rClearAllDirs.setObjectName("rClearAllDirs")
        self.verticalLayout_4.addWidget(self.rClearAllDirs)
        self.lClear = QtWidgets.QListWidget(self.groupBox)
        self.lClear.setMinimumSize(QtCore.QSize(0, 100))
        self.lClear.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.lClear.setObjectName("lClear")
        self.verticalLayout_4.addWidget(self.lClear)
        self.downHL.addWidget(self.groupBox)
        self.verticalLayout_3.addLayout(self.downHL)
        self.tabs.addTab(self.tab_2, "")
        self.horizontalLayout.addWidget(self.tabs)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(Q7FileWindow)
        self.tabs.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(Q7FileWindow)

    def retranslateUi(self, Q7FileWindow):
        _translate = QtCore.QCoreApplication.translate
        Q7FileWindow.setWindowTitle(_translate("Q7FileWindow", "Form"))
        self.cNoLargeData.setToolTip(_translate("Q7FileWindow", "Nodes with large data are read but their data is not"))
        self.cNoLargeData.setText(_translate("Q7FileWindow", "Do not load large data"))
        self.cShowDirs.setText(_translate("Q7FileWindow", "Show directories"))
        self.bCurrent.setToolTip(_translate("Q7FileWindow", "Go to launch directory"))
        self.bBack.setToolTip(_translate("Q7FileWindow", "Go back to parent directory"))
        self.bAction.setText(_translate("Q7FileWindow", "LOAD"))
        self.bClose.setText(_translate("Q7FileWindow", "Cancel"))
        self.tabs.setTabText(self.tabs.indexOf(self.Selection), _translate("Q7FileWindow", "Selection"))
        self.groupBox_4.setTitle(_translate("Q7FileWindow", "Save"))
        self.cOverwrite.setToolTip(_translate("Q7FileWindow", "Overwrite an existing file with new contents"))
        self.cOverwrite.setText(_translate("Q7FileWindow", "Overwrite"))
        self.cDeleteMissing.setToolTip(_translate("Q7FileWindow", "Children found in existing file but not in current tree are removed"))
        self.cDeleteMissing.setText(_translate("Q7FileWindow", "Delete missing"))
        self.cSkipEmpty.setText(_translate("Q7FileWindow", "Skip empty"))
        self.cSaveWithoutLinks.setText(_translate("Q7FileWindow", "Do not save with links"))
        self.groupBox_5.setTitle(_translate("Q7FileWindow", "CHLone options (both load and save)"))
        self.checkBox.setText(_translate("Q7FileWindow", "Trace"))
        self.checkBox_2.setText(_translate("Q7FileWindow", "Debug (quite large output)"))
        self.checkBox_3.setText(_translate("Q7FileWindow", "Send output to:"))
        self.groupBox_3.setTitle(_translate("Q7FileWindow", "Load"))
        self.cNoLargeData_2.setToolTip(_translate("Q7FileWindow", "Nodes with large data are read but their data is not"))
        self.cNoLargeData_2.setText(_translate("Q7FileWindow", "Do not load large data"))
        self.cFollowLinks.setText(_translate("Q7FileWindow", "Follow links"))
        self.cReadOnly.setText(_translate("Q7FileWindow", "Open as read-only"))
        self.cLoadSubPath.setText(_translate("Q7FileWindow", "Load sub-tree with path:"))
        self.cLimitDepth.setText(_translate("Q7FileWindow", "Limit depth to:"))
        self.tabs.setTabText(self.tabs.indexOf(self.tab), _translate("Q7FileWindow", "Load/Save options"))
        self.cActivate.setText(_translate("Q7FileWindow", "Activate directory/file history"))
        self.cAutoDir.setText(_translate("Q7FileWindow", "Auto-Change directory"))
        self.groupBox_2.setTitle(_translate("Q7FileWindow", "Show"))
        self.cShowAll.setText(_translate("Q7FileWindow", "All file extensions"))
        self.__O_filterhdffiles.setText(_translate("Q7FileWindow", ".hdf files"))
        self.__O_filtercgnsfiles.setText(_translate("Q7FileWindow", ".cgns/.adf files"))
        self.cShowOwnExt.setText(_translate("Q7FileWindow", "own extension:"))
        self.groupBox.setTitle(_translate("Q7FileWindow", "Clear history"))
        self.rClearNotFound.setText(_translate("Q7FileWindow", "Not found directory and file entries"))
        self.bClearHistory.setText(_translate("Q7FileWindow", "Clear"))
        self.rClearNoHDF.setText(_translate("Q7FileWindow", "Directory entries without correct file e&xtension"))
        self.rClearSelectedDirs.setText(_translate("Q7FileWindow", "Selected directory entries"))
        self.rClearSelectedFiles.setText(_translate("Q7FileWindow", "Selected file entries"))
        self.rClearAllDirs.setText(_translate("Q7FileWindow", "All director&y and file entries"))
        self.tabs.setTabText(self.tabs.indexOf(self.tab_2), _translate("Q7FileWindow", "History/Filter"))
from . import Res_rc



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
import sys

import linecache
import numpy
import os

from qtpy.QtCore import (Qt, QModelIndex, QAbstractItemModel, QItemSelectionModel)
from qtpy.QtGui import (QIcon, QPixmap, QColor)
from qtpy.QtWidgets import QTreeView

import CGNS.NAV.wmessages as MSG
import CGNS.PAT.cgnskeywords as CGK
import CGNS.PAT.cgnsutils as CGU
import CGNS.VAL.grammars.CGNS_VAL_USER_DEFAULT as CGV
import CGNS.VAL.parse.findgrammar
import CGNS.VAL.parse.messages as CGM
import CGNS.VAL.simplecheck as CGV
from CGNS.NAV.moption import Q7OptionContext as OCTXT
from CGNS.NAV.wfingerprint import Q7FingerPrint


def trace(f):
    def globaltrace(frame, why, arg):
        if why == "call":
            return localtrace
        return None

    def localtrace(frame, why, arg):
        if why == "line":
            # record the file name and line number of every trace
            filename = frame.f_code.co_filename
            lineno = frame.f_lineno

            bname = os.path.basename(filename)
            print("{}({}): {}".format(bname,
                                      lineno,
                                      linecache.getline(filename, lineno)), )
        return localtrace

    def _f(*args, **kwds):
        sys.settrace(globaltrace)
        result = f(*args, **kwds)
        sys.settrace(None)
        return result

    return _f


HIDEVALUE = '@@HIDE@@'
LAZYVALUE = '@@LAZY@@'

(COLUMN_NAME,
 COLUMN_SIDS,
 COLUMN_FLAG_LINK,
 COLUMN_FLAG_SELECT,
 COLUMN_FLAG_CHECK,
 COLUMN_FLAG_USER,
 COLUMN_SHAPE,
 COLUMN_DATATYPE,
 COLUMN_VALUE) = range(9)

COLUMN_LAST = COLUMN_VALUE

COLUMN_ICO = [COLUMN_FLAG_LINK, COLUMN_FLAG_SELECT, COLUMN_FLAG_CHECK,
              COLUMN_FLAG_USER, COLUMN_VALUE]
COLUMN_FLAGS = [COLUMN_FLAG_LINK, COLUMN_FLAG_SELECT, COLUMN_FLAG_CHECK,
                COLUMN_FLAG_USER]

COLUMN_EDIT = [COLUMN_NAME, COLUMN_SIDS, COLUMN_DATATYPE, COLUMN_VALUE]

COLUMN_TITLE = ['-'] * (COLUMN_LAST + 1)

COLUMN_TITLE[COLUMN_NAME] = 'Name'
COLUMN_TITLE[COLUMN_SIDS] = 'SIDS type'
COLUMN_TITLE[COLUMN_FLAG_LINK] = 'L'
COLUMN_TITLE[COLUMN_SHAPE] = 'Shape'
COLUMN_TITLE[COLUMN_FLAG_SELECT] = 'M'
COLUMN_TITLE[COLUMN_FLAG_CHECK] = 'C'
COLUMN_TITLE[COLUMN_FLAG_USER] = 'U'
COLUMN_TITLE[COLUMN_DATATYPE] = 'D'
COLUMN_TITLE[COLUMN_VALUE] = 'Value'

STLKTOPOK = '@@LTOPOK@@'  # top link entry ok
STLKCHDOK = '@@LCHDOK@@'  # child link entry ok
STLKTOPBK = '@@LTOPBK@@'  # broken top link entry
STLKTOPNF = '@@LTOPNF@@'  # top link entry ok not followed
STLKNOLNK = '@@LNOLNK@@'  # no link
STLKIGNOD = '@@LIGNOD@@'  # link ignored by load process

STCHKUNKN = '@@CKUNKN@@'  # unknown
STCHKGOOD = '@@CKGOOD@@'  # good, ok
STCHKWARN = '@@CKWARN@@'  # warning
STCHKFAIL = '@@CKFAIL@@'  # fail, bad
STCHKUSER = '@@CKUSER@@'  # user condition

STCHKLIST = (STCHKUNKN, STCHKGOOD, STCHKWARN, STCHKFAIL, STCHKUSER)

STUSR__ = '@@USR_%s@@'
STUSR_P = '@@USR_%d@@'
STUSR_X = '@@USR_X@@'
STUSR_0 = '@@USR_0@@'
STUSR_1 = '@@USR_1@@'
STUSR_2 = '@@USR_2@@'
STUSR_3 = '@@USR_3@@'
STUSR_4 = '@@USR_4@@'
STUSR_5 = '@@USR_5@@'
STUSR_6 = '@@USR_6@@'
STUSR_7 = '@@USR_7@@'
STUSR_8 = '@@USR_8@@'
STUSR_9 = '@@USR_9@@'
STUSR_A = '@@USR_A@@'
STUSR_B = '@@USR_B@@'
STUSR_C = '@@USR_C@@'
STUSR_D = '@@USR_D@@'
STUSR_E = '@@USR_E@@'
STUSR_F = '@@USR_F@@'

USERSTATES = [STUSR_0, STUSR_1, STUSR_2, STUSR_3, STUSR_4,
              STUSR_5, STUSR_6, STUSR_7, STUSR_8, STUSR_9,
              STUSR_A, STUSR_B, STUSR_C, STUSR_D, STUSR_E, STUSR_F]

STMARK_ON = '@@MARK_ON@@'
STMARKOFF = '@@MARKOFF@@'

STSHRUNKN = '@@SHRUNKN@@'

EDITNODE = '@@NODEDIT@@'
NEWCHILDNODE = '@@NODENEWC@@'
NEWBROTHERNODE = '@@NODENEWB@@'
MARKNODE = '@@NODEMARK@@'
DOWNNODE = '@@NODEDOWN@@'
UPNODE = '@@NODEUP@@'
COPY = '@@NODECOPY@@'
CUT = '@@NODECUT@@'
DELETENODE = '@@NODEDELETE@@'
PASTEBROTHER = '@@NODEPASTEB@@'
PASTECHILD = '@@NODEPASTEC@@'
CUTSELECTED = '@@NODECUTS@@'
PASTEBROTHERSELECTED = '@@NODEPASTEBS@@'
PASTECHILDSELECTED = '@@NODEPASTECS@@'
OPENFORM = '@@OPENFORM@@'
OPENVIEW = '@@OPENVIEW@@'
LOADNODEDATASELECTED = '@@LOADNODEDATASELECTED@@'
RELEASENODEDATASELECTED = '@@RELEASENODEDATASELECTED@@'
LOADNODEDATA = '@@LOADNODEDATA@@'
RELEASENODEDATA = '@@RELEASENODEDATA@@'
OPENLINKEDTOFILE = '@@OPENLINKEDTOFILE@@'
EXPANDSUBTREE = '@@EXPANDSUBTREE@@'
COLLAPSESUBTREE = '@@COLLAPSESUBTREE@@'
EXPANDSUBTREESELECTED = '@@EXPANDSUBTREESELECTED@@'
COLLAPSESUBTREESELECTED = '@@COLLAPSESUBTREESELECTED@@'

DT_LARGE = '@@DATALARGE@@'
DT_LAZY = '@@DATALAZY@@'

USERFLAG_0 = '@@USER0@@'
USERFLAG_1 = '@@USER1@@'
USERFLAG_2 = '@@USER2@@'
USERFLAG_3 = '@@USER3@@'
USERFLAG_4 = '@@USER4@@'
USERFLAG_5 = '@@USER5@@'
USERFLAG_6 = '@@USER6@@'
USERFLAG_7 = '@@USER7@@'
USERFLAG_8 = '@@USER8@@'
USERFLAG_9 = '@@USER9@@'
USERFLAG_A = '@@USERA@@'
USERFLAG_B = '@@USERB@@'
USERFLAG_C = '@@USERC@@'
USERFLAG_D = '@@USERD@@'
USERFLAG_E = '@@USERE@@'
USERFLAG_F = '@@USERF@@'

USERFLAGS = [USERFLAG_0, USERFLAG_1, USERFLAG_2, USERFLAG_3, USERFLAG_4,
             USERFLAG_5, USERFLAG_6, USERFLAG_7, USERFLAG_8, USERFLAG_9,
             USERFLAG_A, USERFLAG_B, USERFLAG_C, USERFLAG_D, USERFLAG_E,
             USERFLAG_F,
             ]

ICONMAPPING = {
    STLKNOLNK: ":/images/icons/empty.png",
    STLKTOPOK: ":/images/icons/link.png",
    STLKCHDOK: ":/images/icons/link-child.png",
    STLKTOPBK: ":/images/icons/link-break.png",
    STLKTOPNF: ":/images/icons/link-error.png",
    STLKIGNOD: ":/images/icons/link-ignore.png",

    STCHKUNKN: ":/images/icons/empty.png",
    STCHKGOOD: ":/images/icons/check-ok.png",
    STCHKFAIL: ":/images/icons/check-fail.png",
    STCHKWARN: ":/images/icons/check-warn.png",

    STUSR_X: ":/images/icons/empty.png",
    STSHRUNKN: ":/images/icons/empty.png",
    STMARKOFF: ":/images/icons/empty.png",
    STMARK_ON: ":/images/icons/mark-node.png",

    STUSR_0: ":/images/icons/user-0.png",
    STUSR_1: ":/images/icons/user-1.png",
    STUSR_2: ":/images/icons/user-2.png",
    STUSR_3: ":/images/icons/user-3.png",
    STUSR_4: ":/images/icons/user-4.png",
    STUSR_5: ":/images/icons/user-5.png",
    STUSR_6: ":/images/icons/user-6.png",
    STUSR_7: ":/images/icons/user-7.png",
    STUSR_8: ":/images/icons/user-8.png",
    STUSR_9: ":/images/icons/user-9.png",
    STUSR_A: ":/images/icons/user-A.png",
    STUSR_B: ":/images/icons/user-B.png",
    STUSR_C: ":/images/icons/user-C.png",
    STUSR_D: ":/images/icons/user-D.png",
    STUSR_E: ":/images/icons/user-E.png",
    STUSR_F: ":/images/icons/user-F.png",

    DT_LARGE: ":/images/icons/data-array-large.png",
    DT_LAZY: ":/images/icons/data-array-lazy.png",
}

KEYMAPPING = {
    MARKNODE: Qt.Key_Space,
    UPNODE: Qt.Key_Up,
    DOWNNODE: Qt.Key_Down,
    OPENFORM: Qt.Key_F,
    OPENVIEW: Qt.Key_W,

    NEWCHILDNODE: Qt.Key_A,
    NEWBROTHERNODE: Qt.Key_Z,
    EDITNODE: Qt.Key_Insert,
    COPY: Qt.Key_C,
    CUT: Qt.Key_X,
    DELETENODE: Qt.Key_Delete,
    PASTECHILD: Qt.Key_Y,
    PASTEBROTHER: Qt.Key_V,
    CUTSELECTED: Qt.Key_X,
    PASTECHILDSELECTED: Qt.Key_Y,
    PASTEBROTHERSELECTED: Qt.Key_V,

    EXPANDSUBTREE: Qt.Key_Plus,
    COLLAPSESUBTREE: Qt.Key_Minus,
    EXPANDSUBTREESELECTED: Qt.Key_Plus,
    COLLAPSESUBTREESELECTED: Qt.Key_Minus,

    LOADNODEDATA: Qt.Key_L,
    RELEASENODEDATA: Qt.Key_R,
    LOADNODEDATASELECTED: Qt.Key_L,
    RELEASENODEDATASELECTED: Qt.Key_R,

    OPENLINKEDTOFILE: Qt.Key_O,

    USERFLAG_0: Qt.Key_0,
    USERFLAG_1: Qt.Key_1,
    USERFLAG_2: Qt.Key_2,
    USERFLAG_3: Qt.Key_3,
    USERFLAG_4: Qt.Key_4,
    USERFLAG_5: Qt.Key_5,
    USERFLAG_6: Qt.Key_6,
    USERFLAG_7: Qt.Key_7,
    USERFLAG_8: Qt.Key_8,
    USERFLAG_9: Qt.Key_9,
    USERFLAG_A: Qt.Key_A,
    USERFLAG_B: Qt.Key_B,
    USERFLAG_C: Qt.Key_C,
    USERFLAG_D: Qt.Key_D,
    USERFLAG_E: Qt.Key_E,
    USERFLAG_F: Qt.Key_F,
}

MODIFIERMAPPING = {
    OPENFORM: (Qt.Key_Control,),
    OPENVIEW: (Qt.Key_Control,),

    EXPANDSUBTREE: (Qt.Key_Control,),
    COLLAPSESUBTREE: (Qt.Key_Control,),
    EXPANDSUBTREESELECTED: (Qt.Key_Control, Qt.Key_Shift),
    COLLAPSESUBTREESELECTED: (Qt.Key_Control, Qt.Key_Shift),

    NEWCHILDNODE: (Qt.Key_Control,),
    NEWBROTHERNODE: (Qt.Key_Control,),
    EDITNODE: (Qt.Key_Control,),
    COPY: (Qt.Key_Control,),
    CUT: (Qt.Key_Control,),
    PASTECHILD: (Qt.Key_Control,),
    PASTEBROTHER: (Qt.Key_Control,),
    CUTSELECTED: (Qt.Key_Control, Qt.Key_Shift),
    PASTECHILDSELECTED: (Qt.Key_Control, Qt.Key_Shift),
    PASTEBROTHERSELECTED: (Qt.Key_Control, Qt.Key_Shift),

    LOADNODEDATA: (Qt.Key_Control,),
    RELEASENODEDATA: (Qt.Key_Control,),
    LOADNODEDATASELECTED: (Qt.Key_Control, Qt.Key_Shift),
    RELEASENODEDATASELECTED: (Qt.Key_Control, Qt.Key_Shift),

    OPENLINKEDTOFILE: (Qt.Key_Control,),
}

EDITKEYMAPPINGS = [
    KEYMAPPING[NEWCHILDNODE],
    KEYMAPPING[NEWBROTHERNODE],
    KEYMAPPING[COPY],
    KEYMAPPING[CUT],
    KEYMAPPING[DELETENODE],
    KEYMAPPING[PASTECHILD],
    KEYMAPPING[PASTEBROTHER],
    KEYMAPPING[CUTSELECTED],
    KEYMAPPING[PASTECHILDSELECTED],
    KEYMAPPING[PASTEBROTHERSELECTED],
]

USERKEYMAPPINGS = [KEYMAPPING[USERFLAG_0], KEYMAPPING[USERFLAG_1],
                   KEYMAPPING[USERFLAG_2], KEYMAPPING[USERFLAG_3],
                   KEYMAPPING[USERFLAG_4], KEYMAPPING[USERFLAG_5],
                   KEYMAPPING[USERFLAG_6], KEYMAPPING[USERFLAG_7],
                   KEYMAPPING[USERFLAG_8], KEYMAPPING[USERFLAG_9],
                   KEYMAPPING[USERFLAG_A], KEYMAPPING[USERFLAG_B],
                   KEYMAPPING[USERFLAG_C], KEYMAPPING[USERFLAG_D],
                   KEYMAPPING[USERFLAG_E], KEYMAPPING[USERFLAG_F],
                   ]

ALLKEYMAPPINGS = [KEYMAPPING[v] for v in KEYMAPPING]


# -----------------------------------------------------------------
# class Q7TreeFilterProxy(QSortFilterProxyModel):
#     def __init__(self,parent):
#         QSortFilterProxyModel.__init__(self,parent)
#         self._treeview=parent.treeview
#         self._control=parent.parent
#         self._wparent=parent
#         self.setDynamicSortFilter(False)
#         self.setFilterRole( Qt.EditRole | Qt.DisplayRole )
#     def lessThan(self,i1,i2):
#         fsort=self.sourceModel().getSortedChildRow
#         c1=fsort(i1.internalPointer().sidsPath())
#         c2=fsort(i2.internalPointer().sidsPath())
#         return c2<c1

# -----------------------------------------------------------------
class Q7TreeView(QTreeView):
    def __init__(self, parent):
        QTreeView.__init__(self, None)
        self._parent = parent
        self._control = None
        self._fgindex = -1
        self.setUniformRowHeights(True)

    @property
    def FG(self):
        return Q7FingerPrint.getByIndex(self._fgindex)

    def isLocked(self):
        return self.FG.isLocked()

    def setControlWindow(self, control, fingerprintindex):
        self._control = control
        self._fgindex = fingerprintindex

    def clearLastEntered(self):
        if self._control is not None:
            return self._control.clearLastEntered()
        return None

    def getLastEntered(self):
        if self._control is not None:
            return self._control.getLastEntered()
        return None

    def setLastEntered(self, ix=None):
        if self._control is not None:
            self._control.setLastEntered(ix)

    def selectionChanged(self, old, new):
        # QTreeView.selectionChanged(self,old,new)
        if old.count():
            n = self.modelData(QModelIndex(old[0].topLeft()))
            self._parent.updateStatus(n)

    def modelData(self, idx):
        if self._control is not None:
            return self._control.modelData(idx)
        return None

    def modelIndex(self, idx):
        if self._control is not None:
            return self._control.modelIndex(idx)
        return None

    def mousePressEvent(self, event):
        self.lastPos = event.globalPos()
        self.lastButton = event.button()
        QTreeView.mousePressEvent(self, event)

    def wink(self, index):
        if self.isExpanded(index):
            self.setExpanded(index, False)
            self.setExpanded(index, True)
        else:
            self.setExpanded(index, True)
            self.setExpanded(index, False)

    def keyPressEvent(self, event):
        kmod = event.modifiers()
        kval = event.key()
        if kval not in ALLKEYMAPPINGS:
            return
        last = self.getLastEntered()
        if last is not None:
            lastpath = last.sidsPath()
            nix = self.model().indexByPath(last.sidsPath())
            pix = self.model().indexByPath(last.parentItem().sidsPath())
            if kval in EDITKEYMAPPINGS:
                if (kmod & Qt.ControlModifier and
                        not kmod & Qt.ShiftModifier):
                    if kval == KEYMAPPING[NEWCHILDNODE]:
                        self.model().newNodeChild(last)
                        nix = self.model().indexByPath(last.sidsPath())
                        self.exclusiveSelectRow(nix)
                    if kval == KEYMAPPING[NEWBROTHERNODE]:
                        self.model().newNodeBrother(last)
                        nix = self.model().indexByPath(last.sidsPath())
                        self.exclusiveSelectRow(nix)
                    if kval == KEYMAPPING[COPY]:
                        self.model().copyNode(last)
                        self.exclusiveSelectRow(nix)
                    if kval == KEYMAPPING[CUT]:
                        if self.model().cutNode(last):
                            self.exclusiveSelectRow(pix)
                        else:
                            self.exclusiveSelectRow(nix)
                    if kval == KEYMAPPING[PASTECHILD]:
                        self.model().pasteAsChild(last)
                        nix = self.model().indexByPath(last.sidsPath())
                        self.exclusiveSelectRow(nix)
                    if kval == KEYMAPPING[PASTEBROTHER]:
                        self.model().pasteAsBrother(last)
                        nix = self.model().indexByPath(last.sidsPath())
                        self.exclusiveSelectRow(nix)
                if (kmod & Qt.ControlModifier and
                        kmod & Qt.ShiftModifier):
                    if kval == KEYMAPPING[CUTSELECTED]:
                        self.model().cutAllSelectedNodes()
                        self.exclusiveSelectRow()
                    if kval == KEYMAPPING[PASTECHILDSELECTED]:
                        self.model().pasteAsChildAllSelectedNodes()
                        self.exclusiveSelectRow()
                    if kval == KEYMAPPING[PASTEBROTHERSELECTED]:
                        self.model().pasteAsBrotherAllSelectedNodes()
                        self.exclusiveSelectRow()
                if kval == KEYMAPPING[DELETENODE]:
                    if self.model().deleteNode(last):
                        self.exclusiveSelectRow(pix)
                    else:
                        self.exclusiveSelectRow(nix)
            # --- same keys different meta
            if ((kval in USERKEYMAPPINGS) and not
            (kmod & Qt.ControlModifier) and not
            (kmod & Qt.ShiftModifier)):
                last.setUserState(kval - 48)
                self.exclusiveSelectRow(nix)
            elif ((kval == KEYMAPPING[EXPANDSUBTREE]) and
                  (kmod & Qt.ControlModifier) and not
                  (kmod & Qt.ShiftModifier)):
                self.expand_sb()
            elif ((kval == KEYMAPPING[COLLAPSESUBTREE]) and
                  (kmod & Qt.ControlModifier) and not
                  (kmod & Qt.ShiftModifier)):
                self.collapse_sb()
            elif ((kval == KEYMAPPING[EXPANDSUBTREESELECTED]) and
                  (kmod & Qt.ControlModifier) and
                  (kmod & Qt.ShiftModifier)):
                self.sexpand_sb()
            elif ((kval == KEYMAPPING[COLLAPSESUBTREESELECTED]) and
                  (kmod & Qt.ControlModifier) and
                  (kmod & Qt.ShiftModifier)):
                self.scollapse_sb()
            elif kval == KEYMAPPING[EDITNODE]:
                if kmod & Qt.ControlModifier:
                    eix = self.model().createIndex(nix.row(), COLUMN_SIDS,
                                                   nix.internalPointer())
                    last.setEditCheck(False)
                    self.edit(eix)
                elif kmod & Qt.ShiftModifier:
                    eix = self.model().createIndex(nix.row(), COLUMN_VALUE,
                                                   nix.internalPointer())
                    last.setEditCheck(False)
                    self.edit(eix)
                else:
                    self.edit(nix)
            elif kval == KEYMAPPING[MARKNODE]:
                self.markNode(last)
                self.exclusiveSelectRow(nix)
            elif kval == KEYMAPPING[UPNODE]:
                if kmod & Qt.ControlModifier:
                    self.upRowLevel(nix)
                elif kmod & Qt.ShiftModifier:
                    self.upRowMarked()
                else:
                    QTreeView.keyPressEvent(self, event)
                    lix = self.modelCurrentIndex()
                    self.exclusiveSelectRow(lix)
            elif kval == KEYMAPPING[DOWNNODE]:
                if kmod & Qt.ControlModifier:
                    self.downRowLevel(nix)
                elif kmod & Qt.ShiftModifier:
                    self.downRowMarked()
                else:
                    QTreeView.keyPressEvent(self, event)
                    lix = self.modelCurrentIndex()
                    self.exclusiveSelectRow(lix)
            elif kval == KEYMAPPING[OPENFORM]:
                self._parent.formview()
                self.exclusiveSelectRow(nix)
            elif kval == KEYMAPPING[OPENVIEW]:
                self._parent.openSubTree()
                self.exclusiveSelectRow(nix)
            else:
                QTreeView.keyPressEvent(self, event)

    def modelCurrentIndex(self):
        idx = self.tryToMapTo(self.currentIndex())
        return idx

    def refreshView(self, ixc=None):
        if ixc is not None and ixc.isValid():
            self.model().refreshModel(ixc)
            return
        ixc = self.model().createIndex(0, 0, None)
        self.model().refreshModel(ixc)

    def markNode(self, node):
        node.switchMarked()
        self.model().updateSelected()
        self.changeRow(node)

    def upRowLevel(self, index):
        self.relativeMoveToRow(-1, index)

    def downRowLevel(self, index):
        self.relativeMoveToRow(+1, index)

    def upRowMarked(self):
        self.changeSelectedMark(-1)

    def downRowMarked(self):
        self.changeSelectedMark(+1)

    def relativeMoveToRow(self, shift, index):
        if (index == -1) or (not index.isValid()):
            index = QModelIndex()
        index = self.tryToMapTo(index)
        row = index.row()
        col = index.column()
        if not index.sibling(row + shift, col).isValid():
            return
        parent = index.parent()
        nix = self.model().index(row + shift, col, parent)
        self.exclusiveSelectRow(nix)

    def changeRow(self, nodeitem):
        pix = self.model().indexByPath(nodeitem.sidsPath()).parent()
        row = pix.row()
        ix1 = self.model().createIndex(row, 0, nodeitem)
        ix2 = self.model().createIndex(row, COLUMN_LAST, nodeitem)
        self.model().dataChanged.emit(ix1, ix2)

    def M(self):
        try:
            m = self.model().sourceModel()
        except AttributeError:
            m = self.model()
        return m

    def tryToMapTo(self, idx):
        if idx.model() is None:
            return idx
        if idx.model() != self.M():
            idx = self.model().mapToSource(idx)
        return idx

    def tryToMapFrom(self, idx):
        if idx.model() is None:
            return idx
        if idx.model() != self.M():
            idx = self.model().mapFromSource(idx)
        return idx

    def selectByPath(self, path):
        npath = CGU.getPathNoRoot(path)
        ix = self.M().indexByPath('/CGNSTree' + npath)
        self.exclusiveSelectRow(ix)

    def exclusiveSelectRow(self, index=-1, setlast=True):
        if index == -1:
            index = QModelIndex()
        elif not index.isValid():
            index = QModelIndex()
        mod = QItemSelectionModel.SelectCurrent | QItemSelectionModel.Rows
        if index.internalPointer() is None:
            return
        self.selectionModel().clearSelection()
        pth = index.internalPointer().sidsPath()
        nix = self.M().indexByPath(pth)
        self.selectionModel().setCurrentIndex(index, mod)
        if setlast:
            self.clearLastEntered()
            self.setLastEntered(index)
        self.scrollTo(index)
        self.refreshView(index)

    def changeSelectedMark(self, delta):
        if self.M()._selected == []:
            return
        sidx = self.M()._selectedIndex
        if self.M()._selectedIndex == -1:
            self.M()._selectedIndex = 0
        elif (delta == -1) and (sidx == 0):
            self.M()._selectedIndex = len(self.M()._selected) - 1
        elif delta == -1:
            self.M()._selectedIndex -= 1
        elif (delta == +1) and (sidx == len(self.M()._selected) - 1):
            self.M()._selectedIndex = 0
        elif delta == +1:
            self.M()._selectedIndex += 1
        if ((self.M()._selectedIndex != -1) and
                (self.M()._selectedIndex < len(self.M()._selected))):
            path = self.M()._selected[self.M()._selectedIndex]
            idx = self.M().match(self.M().index(0, 0, QModelIndex()),
                                 Qt.UserRole,
                                 path,
                                 flags=Qt.MatchExactly | Qt.MatchRecursive)
            if idx[0].isValid():
                self.exclusiveSelectRow(idx[0])

    def doRelease(self):
        self._model = None

    def expand_sb(self):
        ix = self.modelCurrentIndex()
        self._expand_sb(ix)
        self.resizeAll()

    def _expand_sb(self, ix):
        self.expand(ix)
        for c in self.modelData(ix).children():
            ix2 = self.M().indexByPath(c.sidsPath())
            self._expand_sb(ix2)

    def collapse_sb(self):
        ix = self.modelCurrentIndex()
        self._collapse_sb(ix)
        self.resizeAll()

    def _collapse_sb(self, ix):
        self.collapse(ix)
        for c in self.modelData(ix).children():
            ix2 = self.M().indexByPath(c.sidsPath())
            self._collapse_sb(ix2)

    def sexpand_sb(self):
        for pth in self.M().selectedNodes:
            ix = self.M().indexByPath(pth)
            self._expand_sb(ix)
        self.resizeAll()

    def scollapse_sb(self):
        for pth in self.M().selectedNodes:
            ix = self.M().indexByPath(pth)
            self._collapse_sb(ix)
        self.resizeAll()

    def resizeAll(self):
        for n in range(COLUMN_LAST + 1):
            self.resizeColumnToContents(n)


# -----------------------------------------------------------------
def __sortItems(i1, i2):
    c1 = i1.model().getSortedChildRow(i1.sidsPath())
    c2 = i2.model().getSortedChildRow(i2.sidsPath())
    return c1 - c2


# -----------------------------------------------------------------
class Q7TreeItem(object):
    stype = {'MT': 0, 'I4': 4, 'I8': 8, 'R4': 4, 'R8': 8, 'C1': 1, 'LK': 0}
    atype = {'I4': 'int32', 'I8': 'int64',
             'R4': 'float32', 'R8': 'float64',
             'LK': 'S', 'MT': 'S', 'C1': 'S'}
    __lastEdited = None

    def __init__(self, fgprintindex, data, model, tag="", parent=None):
        self._model = model
        self._parentitem = parent
        self._itemnode = data
        self._childrenitems = []
        self._title = COLUMN_TITLE
        self._size = None
        self._fgindex = fgprintindex
        self._log = None
        self._checkable = True
        self._diag = None
        self._states = {'mark': STMARKOFF, 'check': STCHKUNKN,
                        'user': STUSR_X, 'shared': STSHRUNKN}
        self._tag = tag
        self._nodes = 0
        if parent is not None:
            self._path = CGU.getPathNormalize(parent.sidsPath() + '/' + data[0])
            self._path_no_root = CGU.getPathNoRoot(self._path)
            if (model is not None):
                self._model._extension[self._path] = self
                self._nodes = len(self._model._extension)
        else:
            self._path = ''
            self._path_no_root = ''
        self._depth = self._path.count('/')
        if self._path in self.FG.lazy:
            self._lazy = True
        else:
            self._lazy = False
        self.set_sidsIsLinkChild()
        self.set_sidsLinkStatus()
        self.set_sidsDataType()

    @property
    def FG(self):
        return Q7FingerPrint.getByIndex(self._fgindex)

    def orderTag(self):
        return self._tag + "0" * (self.FG.depth * 4 - len(self._tag))

    def updateData(self, data):
        self._itemnode[1] = data

    def sidsIsRoot(self):
        if self._parentitem is None:
            return True
        return False

    def sidsIsCGNSTree(self):
        if self._path == '/CGNSTree':
            return True
        return False

    def sidsParent(self):
        return self._parentitem._itemnode

    def sidsPathSet(self, path):
        del self._model._extension[self._path]
        self._path = path
        self._model._extension[self._path] = self

    def sidsPath(self):
        return self._path

    def sidsNode(self):
        return self._itemnode

    def sidsName(self):
        return self._itemnode[0]

    def sidsNameSet(self, name):
        if not isinstance(name, (str, bytes)):
            return False
        try:
            name = str(name)
        except UnicodeEncodeError:
            return False
        if self._checkable and self._itemnode[0] != name:
            if name == '':
                return False
            if not CGU.checkName(name):
                return False
            if (not CGU.checkDuplicatedName(self.sidsParent(),
                                            name, dienow=False)):
                return False
        self._itemnode[0] = name
        self._checkable = True
        return True

    def sidsValue(self):
        return self._itemnode[1]

    def sidsValueArray(self):
        if isinstance(self._itemnode[1], numpy.ndarray):
            return True
        return False

    def sidsValueFortranOrder(self):
        if self.sidsValueArray():
            return numpy.isfortran(self.sidsValue())
        return False

    def sidsValueEnum(self):
        if self.sidsType() in CGK.cgnsenums:
            return CGK.cgnsenums[self.sidsType()]
        return None

    def trySetAs(self, value, dt):
        try:
            aval = eval(value)
            if aval is None:
                self._itemnode[1] = aval
                return True
            if isinstance(aval, list):
                aval = numpy.array(aval, dtype=dt, order='F')
                self._itemnode[1] = aval
                return True
            else:
                aval = numpy.array([aval], dtype=dt, order='F')
                self._itemnode[1] = aval
                return True
        except:
            pass
        return False

    def sidsValueSet(self, value):
        if value == "":
            return False
        odt = self.sidsDataType()
        if odt == CGK.R4:
            return self.trySetAs(value, 'float32')
        if odt == CGK.R8:
            return self.trySetAs(value, 'float64')
        if odt == CGK.I4:
            return self.trySetAs(value, 'int32')
        if odt == CGK.I8:
            return self.trySetAs(value, 'int64')
        if odt == CGK.MT:
            self._itemnode[1] = None
            return True
        if odt == CGK.C1:
            if not isinstance(value, (str, bytes)):
                return False
            try:
                value = str(value)
            except UnicodeEncodeError:
                return False
            self._itemnode[1] = CGU.setStringAsArray(value)
            return True
        return False

    def sidsChildren(self):
        return self._itemnode[2]

    def sidsType(self):
        return self._itemnode[3]

    def sidsTypePath(self):
        return CGU.getPathAsTypes(self.FG.tree, self._path,
                                  legacy=False)

    def sidsTypeSet(self, value):
        if not isinstance(value, (str, bytes)):
            return False
        try:
            value = str(value)
        except UnicodeEncodeError:
            return False
        if self._checkable and (value not in self.sidsTypeList()):
            return False
        self._itemnode[3] = value
        self._checkable = True
        return True

    def set_sidsDataType(self, all=False):
        if self._itemnode is None:
            return None
        self._datatype = CGU.getValueDataType(self._itemnode)
        return self._datatype

    def sidsDataType(self, all=False):
        if all:
            return CGK.adftypes
        return self._datatype

    def sidsDataTypeSet(self, value):
        if value not in CGK.adftypes:
            print('BAD')
            return False
        odt = self.sidsDataType()
        ndt = value
        adt = Q7TreeItem.atype[ndt]
        oval = self._itemnode[1]
        if (oval is None) and (ndt != CGK.MT):
            if ndt == CGK.C1:
                oval = ""
            else:
                oval = (0,)
        if ndt == CGK.MT:
            nval = None
        else:
            if ((odt not in [CGK.I4, CGK.I8, CGK.R4, CGK.R8]) and
                    (ndt in [CGK.I4, CGK.I8, CGK.R4, CGK.R8])):
                oval = (0,)
            if ndt == CGK.C1:
                oval = CGU.setStringAsArray(oval)
            nval = numpy.array(oval, dtype=adt)
        self._itemnode[1] = nval
        self.set_sidsDataType()
        return True

    def sidsDataTypeSize(self):
        return Q7TreeItem.stype[self._datatype]

    def sidsTypeList(self):
        tlist = CGU.getNodeAllowedChildrenTypes(self._parentitem._itemnode,
                                                self._itemnode)
        return tlist

    def sidsDims(self):
        if isinstance(self.sidsValue(), numpy.ndarray):
            return self.sidsValue().shape
        return (0,)

    def set_sidsLinkStatus(self):
        self._link_status = STLKNOLNK
        self._is_link = False
        if self._path_no_root in [lk[-2] for lk in self.FG.links]:
            self._link_status = STLKTOPOK
            self._is_link = True
        elif self.sidsIsLinkChild():
            self._link_status = STLKCHDOK
            self._is_link = True

    def set_sidsIsLinkChild(self):
        self._is_link_child = False
        pit = self.parentItem()
        if (pit is None): return False
        while not pit.sidsIsRoot():
            if pit.sidsIsLink():
                self._is_link_child = True
                return True
            pit = pit.parentItem()
        return False

    def sidsLinkStatus(self):
        return self._link_status

    def sidsIsLink(self):
        return self._is_link

    def sidsIsLinkChild(self):
        return self._is_link_child

    def sidsLinkFilename(self):
        pth = CGU.getPathNoRoot(self.sidsPath())
        for lk in self.FG.links:
            if pth in lk[-2]:
                return "%s/%s" % (lk[0], lk[1])
        return None

    def sidsLinkValue(self):
        pth = CGU.getPathNoRoot(self.sidsPath())
        for lk in self.FG.links:
            if pth in lk[-2]:
                return "[%s/]%s:%s" % (lk[0], lk[1], lk[2])
        return None

    def sidsRemoveChild(self, node):
        children = self.sidsChildren()
        idx = 0
        while idx < len(children):
            childnode = children[idx]
            if (childnode[0] == node[0]) and (childnode[3] == node[3]):
                break
            idx += 1
        if idx < len(children):
            children.pop(idx)

    def sidsAddChild(self, node):
        if node is None:
            ntype = CGK.UserDefinedData_ts
            name = '{%s#%.3d}' % (ntype, 0)
            newtree = CGU.newNode(name, None, [], ntype)
        else:
            newtree = CGU.nodeCopy(node)
        name = newtree[0]
        ntype = newtree[3]
        parent = self._itemnode
        if parent is None:
            parent = self.FG.tree
        count = 0
        while not CGU.checkDuplicatedName(parent, name, dienow=False):
            count += 1
            name = '{%s#%.3d}' % (ntype, count)
        newtree[0] = name
        parent[2].append(newtree)
        newpath = self.sidsPath() + '/%s' % name
        newrow = self._model.getSortedChildRow(newpath)
        return (newtree, newpath, newrow)

    def addChild(self, item, idx):
        self._childrenitems.insert(idx, item)

    def moveChild(self, item, fromrow, torow):
        self._childrenitems.pop(fromrow)
        self._childrenitems.insert(torow, item)

    def delChild(self, item):
        idx = 0
        while idx < self.childrenCount():
            if item == self._childrenitems[idx]:
                break
            idx += 1
        if idx < self.childrenCount():
            self._childrenitems.pop(idx)

    def children(self):
        return self._childrenitems

    def child(self, row):
        return self._childrenitems[row]

    def childRow(self):
        pth = self.sidsPath()
        parentitem = self.parentItem()
        row = 0
        for child in parentitem._childrenitems:
            if child.sidsPath() == pth:
                return row
            row += 1
        return -1

    def hasChildren(self):
        if self.childrenCount() > 0:
            return True
        return False

    def childrenCount(self):
        return len(self._childrenitems)

    def columnCount(self):
        return COLUMN_LAST + 1

    def setEditCheck(self, check=True):
        self._checkable = check

    def hasEditCheck(self):
        return self._checkable

    def dataRelease(self):
        if self._lazy:
            return
        if self.sidsPath() not in self.FG.lazy:
            self.FG.lazy[self.sidsPath()] = 1
        self._lazy = True
        self.updateData(None)

    def dataLoad(self, partialtree):
        if not self._lazy:
            return
        nodetab = self.sidsValue()
        npath = CGU.getPathNoRoot(self.sidsPath())
        if (nodetab is None) and (partialtree[2] is not None):
            nval = CGU.getNodeByPath(partialtree, npath)
            if nval is not None:
                self.updateData(nval[1])
                if npath in self.FG.lazy:
                    del self.FG.lazy.remove[self.sidsPath()]
                self._lazy = False
        return self

    def hasLazyLoad(self):
        return self._lazy

    def hasValueView(self):
        if self._lazy:
            return False
        if self.sidsValue() is None:
            return False
        if isinstance(self.sidsValue(), numpy.ndarray):
            if not self.sidsValue().shape:
                return False
            try:
                vsize = 1
                for x in self.sidsValue().shape:
                    vsize *= x
            except TypeError:
                print('# CGNS.NAV unexpected (mtree.hasValueView)',
                      self.sidsPath())
            if ((vsize > OCTXT.MaxDisplayDataSize) and
                    (OCTXT.MaxDisplayDataSize > 0)):
                return False
            if self.sidsValue().dtype.kind in ['S', 'a']:
                if len(self.sidsValue().shape) == 1:
                    return True
                if len(self.sidsValue().shape) > 2:
                    return False
                return False
            return True
        return False

    def data(self, column):
        if self._itemnode is None:
            return self._title[column]
        if column == COLUMN_NAME:
            return self.sidsName()
        if column == COLUMN_SIDS:
            return self.sidsType()
        if column == COLUMN_FLAG_LINK:
            return self.sidsLinkStatus()
        if column == COLUMN_SHAPE:
            if self.sidsValue() is None:
                return None
            if not isinstance(self.sidsValue(), numpy.ndarray):
                return None
            if not self.sidsValue().shape:
                return '(0,)'
            return str(self.sidsValue().shape)
        if column == COLUMN_FLAG_SELECT:
            return self._states['mark']
        if column == COLUMN_DATATYPE:
            return self.sidsDataType()
        if column == COLUMN_FLAG_CHECK:
            return self._states['check']
        if column == COLUMN_FLAG_USER:
            return self._states['user']
        if column == COLUMN_VALUE:
            if self._lazy:
                return LAZYVALUE
            if self.sidsValue() is None:
                return None
            if isinstance(self.sidsValue(), numpy.ndarray):
                if not self.sidsValue().shape:
                    vsize = 0
                else:
                    try:
                        vsize = 1
                        for x in self.sidsValue().shape:
                            vsize *= x
                    except TypeError:
                        print('# CGNS.NAV unexpected (mtree.data)', self.sidsPath())
                if ((vsize > OCTXT.MaxDisplayDataSize) and
                        (OCTXT.MaxDisplayDataSize > 0)):
                    return HIDEVALUE
                if self.sidsValue().dtype.kind in ['S', 'a']:
                    if vsize == 0:
                        return ''
                    if len(self.sidsValue().shape) == 1:
                        return self.sidsValue().tostring().decode('ascii')
                    if len(self.sidsValue().shape) > 2:
                        return HIDEVALUE
                    # TODO: use qtextedit for multiple lines
                    # v=self.sidsValue().T
                    # v=numpy.append(v,numpy.array([['\n']]*v.shape[0]),1)
                    # v=v.tostring().decode('ascii')
                    # return v
                    return HIDEVALUE
                if (self.sidsValue().shape == (1,)) and OCTXT.Show1DAsPlain:
                    return str(self.sidsValue()[0])
            return str(self.sidsValue().tolist())[:100]
        return None

    def parentItem(self):
        p = self._parentitem
        return p

    def row(self):
        if self._parentitem:
            return self._parentitem._childrenitems.index(self)
        return 0

    def switchMarked(self):
        if self._states['mark'] == STMARK_ON:
            self._states['mark'] = STMARKOFF
        else:
            self._states['mark'] = STMARK_ON

    def setCheck(self, check):
        if check in STCHKLIST:
            self._states['check'] = check

    def setDiag(self, diag):
        self._diag = diag

    def getDiag(self):
        return self._diag

    def userState(self):
        return self._states['user']

    def setUserStatePrivate(self, s):
        self._states['user'] = STUSR__ % s

    def setUserState(self, s):
        if int(s) in range(10):
            state = STUSR_P % int(s)
        else:
            state = STUSR__ % chr(s + 48)  # hmmm quite tricky
        if self._states['user'] == state:
            self._states['user'] = STUSR_X
        else:
            self._states['user'] = state

    def lastEdited(self):
        return Q7TreeItem.__lastEdited

    def setLastEdited(self):
        Q7TreeItem.__lastEdited = self
        return self

    def doRelease(self):
        Q7TreeItem.__lastEdited = None
        for c in list(self.children()):
            if c is not self:
                c.doRelease()
                if c.children() == []:
                    c._itemnode = None
                    c._model = None
                    self.delChild(c)


SORTTAG = "%.4x"


# -----------------------------------------------------------------
class Q7TreeModel(QAbstractItemModel):
    _icons = {}

    @classmethod
    def initIcons(cls):
        for ik in ICONMAPPING:
            Q7TreeModel._icons[ik] = QIcon(QPixmap(ICONMAPPING[ik]))

    def __init__(self, fgprintindex, parent=None):
        super(Q7TreeModel, self).__init__(parent)
        if Q7TreeModel._icons == {}:
            Q7TreeModel.initIcons()
        self._extension = {}
        self._fgindex = fgprintindex
        self._rootitem = Q7TreeItem(fgprintindex, (None), None)
        self._control = self.FG.control
        self._control.loadOptions()
        self._slist = OCTXT._SortedTypeList
        self._count = 0
        self._movedPaths = {}
        self.parseAndUpdate(self._rootitem,
                            self.FG.tree,
                            QModelIndex(), 0)
        self.FG.model = self
        self._selected = []
        self._selectedIndex = -1

    @property
    def FG(self):
        return Q7FingerPrint.getByIndex(self._fgindex)

    @property
    def selectedNodes(self):
        return self._selected

    def modelReset(self):
        # self.reset()
        self.beginResetModel()
        self.endResetModel()
        self._extension = {}
        self._count = 0
        self._movedPaths = {}
        self._rootitem = Q7TreeItem(self.FG.index, (None), None)
        self.parseAndUpdate(self._rootitem, self.FG.tree, QModelIndex(), 0)
        self._selected = []
        self._selectedIndex = -1

    def nodeFromPath(self, path):
        if path in self._extension:
            return self._extension[path]
        return None

    def modifiedPaths(self, node, parentpath, newname, oldname):
        self._movedPaths = {}
        newpath = parentpath + '/' + newname
        oldpath = parentpath + '/' + oldname
        oldplen = len(oldpath)
        node.sidsPathSet(newpath)
        plist = list(self._selected)
        for p in plist:
            if CGU.hasSameRootPath(oldpath, p):
                self._selected.remove(p)
                self._selected += [str(newpath + p[oldplen:])]
        for p in self._extension:
            if CGU.hasSameRootPath(oldpath, p):
                pn = str(newpath + p[oldplen:])
                nd = self._extension[p]
                nd.sidsPathSet(pn)
                self._movedPaths[p] = pn

    def sortNamesAndTypes(self, paths):
        t = []
        if paths is None:
            return []
        for p in paths:
            n = self.nodeFromPath(p)
            if n is not None:
                t += [(n.orderTag(), p)]
        t.sort()
        return [e[1] for e in t]

    def getSelected(self, noroot=False):
        if noroot:
            return [CGU.getPathNoRoot(pth) for pth in self._selected]
        return self._selected

    def getSelectedZones(self):
        zlist = []
        for pth in self._selected:
            spth = pth.split('/')
            if len(spth) > 3:
                zpth = '/'.join([''] + spth[2:4])
                node = CGU.getNodeByPath(self.FG.tree, zpth)
                if ((node is not None) and
                        (node[3] == CGK.Zone_ts) and
                        (zpth not in zlist)):
                    zlist += [zpth]
            if len(spth) == 3:
                bpth = '/'.join([''] + spth[2:])
                for node in CGU.getChildrenByPath(self.FG.tree, bpth):
                    if node[3] == CGK.Zone_ts:
                        zpth = bpth + '/' + node[0]
                        if zpth not in zlist:
                            zlist += [zpth]
        return zlist

    def getSelectedShortCut(self, ulist=None):
        slist = []
        if ulist is None:
            ulist = self._selected
        for pth in ulist:
            if CGU.hasFirstPathItem(pth):
                pth = CGU.removeFirstPathItem(pth)
            slist += [pth]
        return slist

    def updateSelected(self):
        self._selected = []
        self._selectedIndex = -1
        for k in self._extension:
            if self._extension[k]._states['mark'] == STMARK_ON:
                self._selected += [self._extension[k].sidsPath()]
        self._selected = self.sortNamesAndTypes(self._selected)

    def checkSelected(self):
        return self.checkTree(self.FG.tree, self._selected)

    def checkClear(self):
        for k in self._extension:
            self._extension[k]._states['check'] = STCHKUNKN

    def markExtendToList(self, mlist):
        for k in self._extension:
            if k in mlist:
                self._extension[k]._states['mark'] = STMARK_ON

    def markAll(self):
        for k in self._extension:
            self._extension[k]._states['mark'] = STMARK_ON

    def unmarkAll(self):
        for k in self._extension:
            self._extension[k]._states['mark'] = STMARKOFF

    def swapMarks(self):
        for k in self._extension:
            self._extension[k].switchMarked()

    def columnCount(self, parent):
        if parent.isValid():
            return parent.internalPointer().columnCount()
        else:
            return self._rootitem.columnCount()

    def data(self, index, role):
        if not index.isValid():
            return None
        col = index.column()
        item = index.internalPointer()
        if role == Qt.UserRole:
            return item.sidsPath()
        if role == Qt.ToolTipRole:
            if col == COLUMN_FLAG_LINK:
                lk = item.sidsLinkValue()
                if lk is not None:
                    return lk
            if col == COLUMN_FLAG_CHECK:
                dg = item.getDiag()
                if dg is not None:
                    return dg
            return None
        if role not in [Qt.EditRole, Qt.DisplayRole, Qt.DecorationRole]:
            return None
        if ((role == Qt.DecorationRole) and (col not in COLUMN_ICO)):
            return None
        disp = item.data(col)
        if (col == COLUMN_VALUE) and (role == Qt.DecorationRole):
            if disp == HIDEVALUE:
                return Q7TreeModel._icons[DT_LARGE]
            elif disp == LAZYVALUE:
                return Q7TreeModel._icons[DT_LAZY]
            else:
                return None
        if (col == COLUMN_VALUE) and (role == Qt.DisplayRole):
            if disp in [HIDEVALUE, LAZYVALUE]:
                return None
        # if ((index.column()==COLUMN_FLAG_USER) and (role == Qt.DisplayRole)):
        #     return None
        if disp in ICONMAPPING:
            disp = Q7TreeModel._icons[disp]
        return disp

    def flags(self, index):
        if not index.isValid():
            return Qt.NoItemFlags
        return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable

    def headerData(self, section, orientation, role):
        if ((orientation == Qt.Horizontal) and
                (role == Qt.DisplayRole)):
            return self._rootitem.data(section)
        return None

    def indexByPath(self, path, nosort=False):
        if path in self._movedPaths:
            npath = self._movedPaths[path]
            path = npath
        row = self.getSortedChildRow(path, nosort)
        col = COLUMN_NAME
        ix = self.createIndex(row, col, self.nodeFromPath(path))
        if not ix.isValid():
            return QModelIndex()
        return ix

    def index(self, row, column, parent):
        if not self.hasIndex(row, column, parent):
            return QModelIndex()
        if not parent.isValid():
            parentitem = self._rootitem
        else:
            parentitem = parent.internalPointer()
        childitem = parentitem.child(row)
        if childitem:
            return self.createIndex(row, column, childitem)
        return QModelIndex()

    def parent(self, index):
        if (not index.isValid()) or (index.internalPointer() is None):
            return QModelIndex()
        if index.internalPointer().sidsPath() == '/CGNSTree':
            return QModelIndex()
        childitem = index.internalPointer()
        parentitem = childitem.parentItem()
        if parentitem is None:
            return QModelIndex()
        return self.createIndex(parentitem.row(), 0, parentitem)

    def rowCount(self, parent):
        if parent.column() > 0:
            return 0
        if not parent.isValid():
            parentitem = self._rootitem
        else:
            parentitem = parent.internalPointer()
        if isinstance(parentitem, QModelIndex):
            return 0
        return parentitem.childrenCount()

    def getSortedChildRow(self, path, nosort=False):
        trace = 0
        npath = CGU.getPathNoRoot(path)
        if npath == '/':
            return -1
        targetname = CGU.getPathLeaf(npath)
        parentpath = CGU.getPathAncestor(npath)
        node = CGU.getNodeByPath(self.FG.tree, parentpath)
        if node is None:
            node = self.FG.tree
        row = 0
        if nosort:
            for childnode in node[2]:
                if childnode[0] == targetname:
                    return row
                row += 1
            return -1
        for childnode in CGU.getNextChildSortByType(node, criteria=self._slist):
            if childnode[0] == targetname:
                return row
            row += 1
        return -1

    def setData(self, index, value, role):
        if self.FG.isLocked():
            return
        if ((value is None) or
                (role != Qt.EditRole) or
                (not index.isValid()) or
                (index.column() not in COLUMN_EDIT)):
            return False
        node = index.internalPointer()
        if node.sidsIsLinkChild():
            return False
        node.setLastEdited()
        oldpath = node.sidsPath()
        oldname = node.sidsName()
        oldtype = node.sidsType()
        oldindex = self.indexByPath(node.sidsPath())
        parentpath = node.parentItem().sidsPath()
        fromrow = node.row()
        st = False
        if index.column() == COLUMN_NAME:
            newname = value
            st = node.sidsNameSet(value)
            if st:
                newpath = parentpath + '/' + newname
                torow = self.getSortedChildRow(newpath)
                node.parentItem().moveChild(node, fromrow, torow)
                self.modifiedPaths(node, parentpath, newname, oldname)
                newindex = self.indexByPath(newpath)
                self.rowsMoved.emit(newindex.parent(), fromrow, fromrow,
                                    newindex, torow)
                self.refreshModel(newindex.parent())
                self.refreshModel(newindex)
            else:
                MSG.wError(self._control,
                           0, "Cannot change name of node: %s" % oldpath,
                           "The new name [%s] is rejected" % newname)
        if index.column() == COLUMN_SIDS:
            newtype = value
            if not node.hasEditCheck() or (newtype in CGK.cgnstypes):
                st = node.sidsTypeSet(newtype)
            if st:
                torow = self.getSortedChildRow(oldpath)
                node.parentItem().moveChild(node, fromrow, torow)
                newindex = self.indexByPath(oldpath)
                self.rowsMoved.emit(newindex.parent(), fromrow, fromrow,
                                    newindex, torow)
                self.refreshModel(newindex.parent())
                self.refreshModel(newindex)
            else:
                MSG.wError(self._control,
                           0, "Cannot change SIDS type of node: %s" % oldpath,
                           "The new type [%s] is rejected" % newtype)
        if index.column() == COLUMN_VALUE:
            st = node.sidsValueSet(value)
            if not st:
                MSG.wError(self._control,
                           0, "Cannot change value of node: %s" % oldpath,
                           "The value is rejected")
        if index.column() == COLUMN_DATATYPE:
            st = node.sidsDataTypeSet(value)
        if st:
            self.FG.addTreeStatus(Q7FingerPrint.STATUS_MODIFIED)
        return st

    def removeItem(self, parentitem, targetitem, row):
        parentindex = self.indexByPath(parentitem.sidsPath())
        path = targetitem.sidsPath()
        self.beginRemoveRows(parentindex, row, row)
        parentitem.delChild(targetitem)
        del self._extension[path]
        self.endRemoveRows()

    def removeItemTree(self, nodeitem):
        self.parseAndRemove(nodeitem)
        row = nodeitem.childRow()
        self.removeItem(nodeitem.parentItem(), nodeitem, row)

    def parseAndRemove(self, parentitem):
        if not parentitem.hasChildren():
            return
        while parentitem.childrenCount() != 0:
            r = parentitem.childrenCount()
            child = parentitem.child(r - 1)
            self.parseAndRemove(child)
            self.removeItem(parentitem, child, r - 1)

    def itemNew(self, fg, nt, md, tg, pi):
        return Q7TreeItem(fg, nt, md, tg, pi)

    def parseAndUpdate(self, parentItem, node, parentIndex, row, parenttag=""):
        self._count += 1
        tag = parenttag + SORTTAG % self._count
        newItem = self.itemNew(self.FG.index, (node), self, tag, parentItem)
        self.FG.depth = max(newItem._depth, self.FG.depth)
        self.FG.nodes = max(newItem._nodes, self.FG.nodes)
        if not parentIndex.isValid():
            parentIndex = self.createIndex(0, 0, self._rootitem)
        self.rowsAboutToBeInserted.emit(parentIndex, row, row)
        self.beginInsertRows(parentIndex, row, row)
        parentItem.addChild(newItem, row)
        self.endInsertRows()
        self.rowsInserted.emit(parentIndex, row, row)
        newIndex = self.createIndex(row, 0, newItem)
        crow = 0
        for childnode in CGU.getNextChildSortByType(node, criteria=self._slist):
            c = self.parseAndUpdate(newItem, childnode, newIndex, crow, tag)
            self.FG.depth = max(c._depth, self.FG.depth)
            self.FG.nodes = max(c._nodes, self.FG.nodes)
            crow += 1
        self.FG.refreshScreen()
        return newItem

    def refreshModel(self, nodeidx):
        self.dataChanged.emit(QModelIndex(), QModelIndex())
        if not nodeidx.isValid():
            return
        row = nodeidx.row()
        dlt = 2
        parentidx = nodeidx.parent()
        maxrow = self.rowCount(parentidx)
        row1 = min(0, abs(row - dlt))
        row2 = min(row + dlt, maxrow)
        ix1 = self.createIndex(row1, 0, parentidx.internalPointer())
        ix2 = self.createIndex(row2, COLUMN_LAST, parentidx.internalPointer())
        if ix1.isValid() and ix2.isValid():
            self.dataChanged.emit(ix1, ix2)
        self.layoutChanged.emit()

    def newNodeChild(self, nodeitem):
        if nodeitem is None:
            return
        nix = self.indexByPath(nodeitem.sidsPath())
        (ntree, npath, nrow) = nodeitem.sidsAddChild(None)
        self.parseAndUpdate(nodeitem, ntree, nix, nrow, nodeitem._tag)
        nix = self.indexByPath(nodeitem.sidsPath())
        pix = self.indexByPath(CGU.getPathAncestor(npath))
        cix = self.indexByPath(npath)
        self.refreshModel(pix)
        self.refreshModel(nix)
        self.refreshModel(cix)
        self.FG.addTreeStatus(Q7FingerPrint.STATUS_MODIFIED)

    def newNodeBrother(self, nodeitem):
        if nodeitem is None:
            return
        nix = self.indexByPath(nodeitem.sidsPath())
        self.newNodeChild(nix.parent().internalPointer())
        self.FG.addTreeStatus(Q7FingerPrint.STATUS_MODIFIED)

    def copyNodeRaw(self, node):
        if node is None:
            return
        # deep copy here
        self._control.copyPasteBuffer = CGU.nodeCopy(node)
        self._control.clearOtherSelections()

    def copyNode(self, nodeitem):
        if nodeitem is None:
            return
        # deep copy here
        self._control.copyPasteBuffer = CGU.nodeCopy(nodeitem._itemnode)
        self._control.clearOtherSelections()

    def cutAllSelectedNodes(self):
        for pth in self._selected:
            nodeitem = self.nodeFromPath(pth)
            self.cutNode(nodeitem)
        self._control.clearOtherSelections()

    def cutNode(self, nodeitem):
        if nodeitem is None:
            return False
        if nodeitem.sidsIsLink() or nodeitem.sidsIsLinkChild():
            return False
        self._control.copyPasteBuffer = CGU.nodeCopy(nodeitem._itemnode)
        parentitem = nodeitem.parentItem()
        path = CGU.getPathAncestor(nodeitem.sidsPath())
        self.removeItemTree(nodeitem)
        pix = self.indexByPath(path)
        parentitem.sidsRemoveChild(self._control.copyPasteBuffer)
        self.refreshModel(pix)
        self.FG.addTreeStatus(Q7FingerPrint.STATUS_MODIFIED)
        self._control.clearOtherSelections()
        return True

    def deleteNode(self, nodeitem):
        if nodeitem is None:
            return False
        if nodeitem.sidsIsLink() or nodeitem.sidsIsLinkChild():
            return False
        _tmpCopy = CGU.nodeCopy(nodeitem._itemnode)
        parentitem = nodeitem.parentItem()
        path = CGU.getPathAncestor(nodeitem.sidsPath())
        self.removeItemTree(nodeitem)
        pix = self.indexByPath(path)
        parentitem.sidsRemoveChild(_tmpCopy)
        self.refreshModel(pix)
        self.FG.addTreeStatus(Q7FingerPrint.STATUS_MODIFIED)
        self._control.clearOtherSelections()
        return True

    def pasteAsChildAllSelectedNodes(self):
        for pth in self._selected:
            nodeitem = self.nodeFromPath(pth)
            self.pasteAsChild(nodeitem)

    def pasteAsChild(self, nodeitem):
        if nodeitem is None:
            return False
        if self._control.copyPasteBuffer is None:
            return False
        if nodeitem.sidsIsLink() or nodeitem.sidsIsLinkChild():
            return
        # Dirty patch
        # Prevent copying inside subtree based on node name
        if self._control.copyPasteBuffer[0] in nodeitem.sidsPath():
            # TODO
            # Would be nice to compare origin path of PasteBuffer
            # either way we should check path of nodes with same name
            # to ensure they are different !
            return False
        nix = self.indexByPath(nodeitem.sidsPath())
        (ntree, npath, nrow) = nodeitem.sidsAddChild(self._control.copyPasteBuffer)
        self.parseAndUpdate(nodeitem, ntree, nix, nrow, nodeitem._tag)
        nix = self.indexByPath(nodeitem.sidsPath())
        pix = self.indexByPath('/CGNSTree' + CGU.getPathAncestor(npath))
        cix = self.indexByPath(npath)
        self.refreshModel(pix)
        self.refreshModel(nix)
        self.refreshModel(cix)
        self.FG.addTreeStatus(Q7FingerPrint.STATUS_MODIFIED)
        return True

    def pasteAsBrotherAllSelectedNodes(self):
        for pth in self._selected:
            nodeitem = self.nodeFromPath(pth)
            self.pasteAsBrother(nodeitem)

    def pasteAsBrother(self, nodeitem):
        if nodeitem is None:
            return False
        if nodeitem.sidsIsLink() or nodeitem.sidsIsLinkChild():
            return
        nix = self.indexByPath(nodeitem.sidsPath())
        self.pasteAsChild(nix.parent().internalPointer())
        self.FG.addTreeStatus(Q7FingerPrint.STATUS_MODIFIED)
        return True

    def dataLoadSelected(self, single=None):
        udict = {}
        if single is None:
            pthlist = self._selected
        else:
            pthlist = [single]
        for pth in pthlist:
            nodeitem = self.nodeFromPath(pth)
            udict[CGU.getPathNoRoot(nodeitem.sidsPath())] = nodeitem.sidsValue()
        if udict:
            (t, l, p) = self.FG.updateNodeData(udict)
        for pth in pthlist:
            nodeitem = self.nodeFromPath(pth)
            nodeitem.dataLoad(t)

    def dataReleaseSelected(self, single=None):
        if single is None:
            pthlist = self._selected
        else:
            pthlist = [single]
        for pth in pthlist:
            nodeitem = self.nodeFromPath(pth)
            nodeitem.dataRelease()

    def checkTree(self, T, pathlist):
        self.FG.pushGrammarPaths()
        modset = set()
        for tag in self.FG.nextGrammarTag():
            modset.add(CGNS.VAL.parse.findgrammar.importUserGrammars(tag))
        self.FG.popGrammarPaths()
        for mod in modset:
            if mod is None:
                checkdiag = CGV.CGNS_VAL_USER_Checks(None)
            else:
                checkdiag = mod.CGNS_VAL_USER_Checks(None)
            checkdiag.checkTree(T, False)
        if pathlist == []:
            pathlist = list(self._extension)
        for path in pathlist:
            pth = CGU.getPathNoRoot(path)
            if pth in checkdiag.log:
                item = self._extension[path]
                stat = checkdiag.log.getWorstDiag(pth)
                if stat == CGM.CHECK_NONE:
                    item.setCheck(STCHKUNKN)
                if stat == CGM.CHECK_GOOD:
                    item.setCheck(STCHKGOOD)
                if stat == CGM.CHECK_FAIL:
                    item.setCheck(STCHKFAIL)
                    s = ""
                    for e in checkdiag.log[pth]:
                        s += checkdiag.log.message(e) + '\n'
                    item.setDiag(s[:-1])
                if stat == CGM.CHECK_WARN:
                    item.setCheck(STCHKWARN)
                    s = ""
                    for e in checkdiag.log[pth]:
                        s += checkdiag.log.message(e) + '\n'
                    item.setDiag(s[:-1])
                if stat == CGM.CHECK_USER:
                    item.setCheck(STCHKUSER)
                    s = ""
                    for e in checkdiag.log[pth]:
                        s += checkdiag.log.message(e) + '\n'
                    item.setDiag(s[:-1])
        return checkdiag.log

    def hasUserColor(self, k):
        cl = OCTXT.UserColors
        try:
            c = k[-3]
            if cl[int(c)] is None:
                return False
        except:
            return False
        return True

    def getUserColor(self, k):
        cb = Qt.black
        cl = OCTXT.UserColors
        c = int(k[-3])
        return QColor(cl[c])

    def doRelease(self):
        self._rootitem.doRelease()
        # self._rootitem=None
        self._control = None
        self._selected = None
        self.extension = None
        Q7TreeItem._icons = {}
        # self.reset()
        self.beginResetModel()
        self.endResetModel()

# -----------------------------------------------------------------



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
# Posted by Cacasodo (http://www.techanswerguy.com)
# creating a transparent favicon.ico with Gimp
#I'm always forgetting the steps to do this properly. Here you go:
#1) open a favorite image that you'd like to turn into a favicon to appear in your browser's location bar
#2) click Image->Mode->RGB
#3) using the erase tool, erase any areas of the graphic that you'd like to be transparent
#4) using Image->Scale, resize the image to 32x32, the proper size for a favicon
#5) first save the file as a GIF
#- Gimp will popup a dialog that says "GIF can only handle grayscale or indexed images"
#6) select * Convert to Indexed using default settings
#7) click Export
#8) save a copy as .ico. It will give you a number of choices for bits/pixel:
#1 bpp, 1 bit alpha, 2-slot palette
#4 bpp, 1 bit alpha, 16-slot palette
#8 bpp, 1 bit alpha, 256-slot palette
#32 bpp, 8 bit alpha, no palette
#
#Choose 4bpp. However, I have seen with some images that 8bpp needs to be used.
#9) upload to your website and enjoy!
import sys
import glob

from cx_Freeze import setup, Executable
import cx_Freeze
print(cx_Freeze.__file__)

df=glob.glob('demo/*')+glob.glob('demo/124Disk-PASS/*')+glob.glob('demo/124Disk-FAIL/*')+glob.glob('demo/DPW5/*')
demo_files=[(f,f) for f in df]

# Dependencies are automatically detected, but it might need fine tuning.
build_exe_options = {
"packages": ["os","sys","numpy","vtk","qtpy","qtpy.QtCore","qtpy.QtGui","qtpy.QtWidgets",
             "CGNS.NAV","CGNS.MAP","CGNS.PAT","CGNS.VAL","CGNS.APP",
             "numpy.core.multiarray"
], 
"excludes": ["tkinter","tcl","tk8.5","PySide","PySide.QtCore","PySide.QtGui",
],
"include_files": ["license.txt",
r"C:\Users\Public\Python3.6\Lib\site-packages\pyCGNS-5.3.0-py3.6-win-amd64.egg",
#r"D:\poinot\AppData\Local\Continuum\Anaconda\Lib\site-packages\pyCGNS-5.3.0-py3.6-win-amd64.egg",
#r"D:\poinot\AppData\Local\Continuum\Anaconda\Lib",
#r"D:\poinot\AppData\Local\Continuum\Anaconda\DLLs",
"lib/pyCGNS.ico",
"lib/pyCGNS-wizard.bmp",
"lib/pyCGNS-small.ico",
"lib/pyCGNS-wizard-small.bmp",                  
]+demo_files,
"add_to_path":True,
}

# GUI applications require a different base on Windows (the default is for a
# console application).
base = None
if (sys.platform == "win32"):
    base = "Win32GUI"

# --- pyCGNSconfig search
import os
sys.path=[os.path.abspath(os.path.normpath('./lib'))]+sys.path
import setuputils

# ---

setup(  
name = "pyCGNS",
version = '4.6',
description  = "pyCGNS - CGNS/Python trees navigator and editor",
author       = "Marc Poinot",
author_email = "marc.poinot@safrangroup.com",
license      = "LGPL 2",
options = {"build_exe": build_exe_options,"build_msi": build_exe_options},
           executables = [Executable("CGNS/App/tools/cg_look",base=base)])

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#
import os
import sys
import string
import argparse
import glob
import re

from setuptools import setup, Extension
from pathlib import Path

# in the case the default python/distutils compiler fails with mpi, set
# this variable. You should check this compiler is the one used for HDF5 prod
# in the share/cmake config of your HDF5 installation.
CYTHON_COMPILER_FOR_MAP = "mpicc"

# --- get overall configuration
sys.path = ["{}/lib".format(os.getcwd())] + sys.path
from setuputils import (
    MAJOR_VERSION,
    MINOR_VERSION,
    HAS_MSW,
    log,
    line,
    is_windows,
    fix_path,
    search,
    clean,
    touch,
    ConfigException,
    updateVersionInFile,
    installConfigFiles,
)


line("pyCGNS v{}.{} install".format(MAJOR_VERSION, MINOR_VERSION))
line()

doc1 = """
  pyCGNS installation setup 
  - Usual python setup options are unchanged (build, install, help...)
  - The recommanded way to build is to set your shell environment with
    your own PATH, LD_LIBRARY_PATH and PYTHONPATH so that the setup would
    find all expected ressources itself.
  - You can either use the command line args as described below or edit
    the setup_userConfig.py with the values you want.

  All packages are installed if all expected dependancies are found.
  See doc for more installation details and depend:
  https://pycgns.github.io/install.html

  MPI: using HDF5 with parallel features adds dependancies on mpi. The
  simple way to resolve these deps is to force mpicc instead of cc:

  CC=mpicc python setup.py build

  or edit the pyCGNS/stup.py file and change CYTHON_COMPILER_FOR_MAP

  ** IMPORTANT WARNING **
  The *install* command runs the *build* as first step.
  If you run first the *build* with specific options
  you *SHOULD* add these options again in the *install*
  command line unless you will have a *NEW* build.

"""

doc2 = """
  Examples:

  1. The best way is to let setup find out required stuff, build and
     install. This would need write access to python installation dirs:

  python setup.py install

  2. You can build and install in two separate commands

  python setup.py build
  python setup.py install

  3. Specific paths would be used prior to any automatic detection:

  python setup.py build --includes=/usr/local/includes:/home/tools/include

  4. Installation to a local directory (usual setup pattern)

  python setup.py install --prefix=/home/myself/install

"""


def str2bool(value):
    if value.lower() in ("yes", "true", "t", "y", "1"):
        return True
    elif value.lower() in ("no", "false", "f", "n", "0"):
        return False
    else:
        raise argparse.ArgumentTypeError("Boolean value expected.")


pr = argparse.ArgumentParser(
    description=doc1,
    epilog=doc2,
    formatter_class=argparse.RawDescriptionHelpFormatter,
    usage="python %(prog)s [options] file1 file2 ...",
)

pr.add_argument(
    "-I",
    "--includes",
    dest="incs",
    help="list of paths for include search ( : separated), order is significant and is kept unchanged",
)
pr.add_argument(
    "-L",
    "--libraries",
    dest="libs",
    help="list of paths for libraries search ( : separated), order is significant and is kept unchanged",
)
pr.add_argument("-U", "--update", action="store_true", help="update version (dev only)")
pr.add_argument(
    "-F", "--force", action="store_false", help="skip all .pyx files regeneration"
)
pr.add_argument(
    "-A",
    "--alternate",
    action="store_true",
    help="use full h5py CGNS/HDF5 interface (ongoing work)",
)

modules = {"app": True, "map": True, "pat": True, "val": True, "nav": True}

for name, val in modules.items():
    pr.add_argument(
        "--" + name,
        type=str2bool,
        default=val,
        help="enable/disable building of CGNS." + name.upper(),
    )

for hstr in ["--help", "-h", "help"]:
    if hstr in sys.argv:
        pr.print_help()
        sys.exit(1)

# Remove modules from command-line arguments
pr1 = argparse.ArgumentParser()
for name, val in modules.items():
    pr1.add_argument("--" + name, type=str2bool, default=val)

args1, unknown = pr1.parse_known_args()
sys.argv = sys.argv[:1] + unknown

if "help" in unknown:
    pr.print_help()
    sys.exit(1)

try:
    os.makedirs("./build/lib/CGNS")
except OSError:
    pass

log("using Python from {}".format(sys.prefix))

if HAS_MSW:
    log("found Windows platform")
else:
    log("found Unix platform")

APP = args1.app
MAP = args1.map
PAT = args1.pat
VAL = args1.val
NAV = args1.nav

ALL_PACKAGES = []
ALL_SCRIPTS = []
ALL_EXTENSIONS = []
OTHER_INCLUDES_PATHS = []
OTHER_LIBRARIES_PATHS = []
EXTRA_DEFINE_MACROS = []

if HAS_MSW:
    EXTRA_DEFINE_MACROS = [("_HDF5USEDLL_", None), ("H5_BUILT_AS_DYNAMIC_LIB", None)]

module_logs = []
incs = []
libs = []

args, unknown = pr.parse_known_args()

if "install" in unknown:
    args.force = False

if args.incs is not None:
    incs = [os.path.expanduser(path) for path in args.incs.split(os.path.pathsep)]
if args.libs is not None:
    libs = [os.path.expanduser(path) for path in args.libs.split(os.path.pathsep)]

if args.alternate:
    deps = ["Cython", "h5py", "numpy", "vtk", "qtpy"]
else:
    deps = ["Cython", "HDF5", "numpy", "vtk", "qtpy"]

RUN_REQUIRES = ["numpy", "future"]
SETUP_REQUIRES = RUN_REQUIRES + ["cython>=0.25", "pkgconfig"]

# Remove crashing deps test
if "sdist" in sys.argv:
    deps = ["Cython", "numpy", "vtk", "qtpy"]

# Dirty patch
# Get required EGG if needed
import setuptools.dist

dist = setuptools.dist.Distribution(dict({"setup_requires": SETUP_REQUIRES}))
try:
    (CONFIG, status) = search(incs, libs, deps=deps)
except ConfigException as e:
    log("***** Cannot build pyCGNS without: {}".format(e))
    sys.exit(1)

# Fake HDF5 Config
if "sdist" in sys.argv:
    CONFIG.HDF5_HST = 1
    CONFIG.HDF5_H64 = 1
    CONFIG.HDF5_HUP = 1
    CONFIG.HDF5_VERSION = "1.10.4"
    CONFIG.HDF5_PARALLEL = 0
    CONFIG.USE_COMPACT_STORAGE = 1


line()

new_args = []

for arg in sys.argv:
    if (
        not ("-I=" in arg or "--includes=" in arg)
        and not ("-U" in arg or "--update" in arg)
        and not ("-A" in arg or "--alternate" in arg)
        and not ("-F" in arg or "--force" in arg)
        and not ("-L=" in arg or "--libraries=" in arg)
    ):
        new_args += [arg]
sys.argv = new_args

if args.update:
    # os.system('hg parents --template="{rev}\n" > %s/revision.tmp' \
    #          % CONFIG.PRODUCTION_DIR)
    # Quick and dirty revision, should use git describe --always instead
    os.system(
        r"git rev-list --count HEAD> {}/revision.tmp".format(CONFIG.PRODUCTION_DIR)
    )
    updateVersionInFile(fix_path("./lib/pyCGNSconfig_default.py"), CONFIG)


def hasToGenerate(source, destination, force):
    return (
        force
        or not os.path.exists(destination)
        or os.path.getmtime(source) > os.path.getmtime(destination)
    )


def resolveVars(filename, confvalues, force):
    if hasToGenerate(filename + ".in", filename, force):
        with open("{}.in".format(filename), "r") as fg1:
            l1 = fg1.readlines()
        l2 = [ll.format(**confvalues) for ll in l1]
        with open(filename, "w") as fg2:
            fg2.writelines(l2)


if args.force:
    print("Generation of pyx not skipped")
else:
    print("Skipping pyx generation")

# -------------------------------------------------------------------------
if APP:
    slist = [
        "cg_grep",
        "cg_list",
        "cg_link",
        "cg_iges",
        "cg_diff",
        "cg_checksum",
        "cg_gather",
        "cg_scatter",
        "cg_dump",
        "cg_scan",
    ]
    if NAV:
        slist += ["cg_look"]

    ALL_SCRIPTS += ["CGNS/APP/tools/%s" % f for f in slist]

    ALL_EXTENSIONS += [
        Extension(
            "CGNS.APP.lib.arrayutils",
            ["CGNS/APP/lib/arrayutils.pyx", "CGNS/APP/lib/hashutils.c"],
            include_dirs=CONFIG.INCLUDE_DIRS
            + OTHER_INCLUDES_PATHS
            + [
                "CGNS/APP/lib",
            ],
            extra_compile_args=[],
        )
    ]
    ALL_PACKAGES += [
        "CGNS.APP",
        "CGNS.APP.lib",
        "CGNS.APP.tools",
        "CGNS.APP.examples",
        "CGNS.APP.misc",
        "CGNS.APP.test",
    ]
    module_logs.append("APP   add  build")
else:
    module_logs.append("APP   skip build *")

# -------------------------------------------------------------------------
if MAP:
    if not CONFIG.HAS_H5PY:
        # generate files
        # CHLone_config.h.in -> CHLone_config.h
        # pyCHLone.pyx.in -> pyCHLone.pyx
        #
        # --- config values
        hdfplib = CONFIG.HDF5_PATH_LIBRARIES
        hdflib = CONFIG.HDF5_LINK_LIBRARIES
        hdfpinc = CONFIG.HDF5_PATH_INCLUDES
        include_dirs = ["."] + hdfpinc + CONFIG.INCLUDE_DIRS + OTHER_INCLUDES_PATHS
        library_dirs = hdfplib
        optional_libs = hdflib
        extra_compile_args = CONFIG.HDF5_EXTRA_ARGS
        extra_define_macro = EXTRA_DEFINE_MACROS

        conf = {
            "CHLONE_HAS_PTHREAD": 1,
            "CHLONE_HAS_REGEXP": 1,
            "CHLONE_PRINTF_TRACE": 0,
            "CHLONE_ON_WINDOWS": HAS_MSW,
            "CHLONE_H5CONF_STD": CONFIG.HDF5_HST,
            "CHLONE_H5CONF_64": CONFIG.HDF5_H64,
            "CHLONE_H5CONF_UP": CONFIG.HDF5_HUP,
            "CHLONE_USE_COMPACT_STORAGE": CONFIG.USE_COMPACT_STORAGE,
            "HDF5_VERSION": CONFIG.HDF5_VERSION,
            "CHLONE_INSTALL_LIBRARIES": "",
            "CHLONE_INSTALL_INCLUDES": "",
        }

        depfiles = ["CGNS/MAP/CHLone_config.h", "CGNS/MAP/EmbeddedCHLone.pyx"]

        EXTRA_MAP_COMPILE_ARGS = ""

        resolveVars(fix_path(depfiles[0]), conf, args.force)
        resolveVars(fix_path(depfiles[1]), conf, args.force)
        library_dirs = [l for l in library_dirs if l != ""]

        # hack: actually shoudl read hdf5/cmake config to get true compiler...
        if CONFIG.HDF5_PARALLEL:
            os.environ["CC"] = CYTHON_COMPILER_FOR_MAP

        ALL_EXTENSIONS += [
            Extension(
                "CGNS.MAP.EmbeddedCHLone",
                [
                    "CGNS/MAP/EmbeddedCHLone.pyx",
                    "CGNS/MAP/SIDStoPython.c",
                    "CGNS/MAP/l3.c",
                    "CGNS/MAP/error.c",
                    "CGNS/MAP/linksearch.c",
                    "CGNS/MAP/sha256.c",
                ],
                include_dirs=include_dirs,
                library_dirs=library_dirs,
                libraries=optional_libs,
                depends=depfiles,
                define_macros=extra_define_macro,
                extra_compile_args=extra_compile_args,
            )
        ]

    ALL_PACKAGES += ["CGNS.MAP", "CGNS.MAP.test"]
    module_logs.append("MAP   add  build")
else:
    module_logs.append("MAP   skip build *")

# -------------------------------------------------------------------------
if VAL:
    ALL_PACKAGES += [
        "CGNS.VAL",
        "CGNS.VAL.grammars",
        "CGNS.VAL.suite",
        "CGNS.VAL.suite.SIDS",
        "CGNS.VAL.parse",
        "CGNS.VAL.test",
    ]
    ALL_SCRIPTS += ["CGNS/VAL/CGNS.VAL"]

    if args.force:
        touch("CGNS/VAL/grammars/CGNS_VAL_USER_SIDS_.pyx")
        touch("CGNS/VAL/grammars/etablesids.pyx")
        touch("CGNS/VAL/grammars/valutils.pyx")

    ALL_EXTENSIONS += [
        Extension(
            "CGNS.VAL.grammars.CGNS_VAL_USER_SIDS_",
            ["CGNS/VAL/grammars/CGNS_VAL_USER_SIDS_.pyx"],
            include_dirs=CONFIG.INCLUDE_DIRS,
            extra_compile_args=[],
        )
    ]
    ALL_EXTENSIONS += [
        Extension(
            "CGNS.VAL.grammars.etablesids",
            ["CGNS/VAL/grammars/etablesids.pyx"],
            include_dirs=CONFIG.INCLUDE_DIRS,
            extra_compile_args=[],
        )
    ]
    ALL_EXTENSIONS += [
        Extension(
            "CGNS.VAL.grammars.valutils",
            ["CGNS/VAL/grammars/valutils.pyx"],
            include_dirs=CONFIG.INCLUDE_DIRS,
            extra_compile_args=[],
        )
    ]

    module_logs.append("VAL   add  build")
else:
    module_logs.append("VAL   skip build *")

# -------------------------------------------------------------------------
if PAT:
    # if CONFIG.HAS_CYTHON:
    #  ALL_EXTENSIONS+=[ Extension('CGNS.PAT.cgnsutils',
    #                              ['CGNS/PAT/cgnsutils.pyx'],
    #                              include_dirs = CONFIG.INCLUDE_DIRS) ]
    ALL_PACKAGES += ["CGNS.PAT", "CGNS.PAT.SIDS", "CGNS.PAT.test"]
    module_logs.append("PAT   add  build")
else:
    module_logs.append("PAT   skip build *")

# -------------------------------------------------------------------------
if NAV and CONFIG.HAS_QTPY:
    cui = CONFIG.COM_UIC
    crc = CONFIG.COM_RCC
    ccy = CONFIG.COM_CYTHON

    fakefile = "./CGNS/NAV/fake.pxi"
    if args.force:
        touch(fakefile)

    modnamelist = [
        "Q7TreeWindow",
        "Q7DiffWindow",
        "Q7MergeWindow",
        "Q7ControlWindow",
        "Q7OptionsWindow",
        "Q7FormWindow",
        "Q7FileWindow",
        "Q7QueryWindow",
        "Q7SelectionWindow",
        "Q7InfoWindow",
        "Q7DiagWindow",
        "Q7LinkWindow",
        "Q7HelpWindow",
        "Q7ToolsWindow",
        "Q7PatternWindow",
        "Q7AnimationWindow",
        "Q7MessageWindow",
        "Q7LogWindow",
    ]
    if CONFIG.HAS_VTK:
        modnamelist += ["Q7VTKWindow"]
    modgenlist = []
    modextlist = []
    mfile_list = [
        "mtree",
        "mquery",
        "mcontrol",
        "mtable",
        "mpattern",
        "diff",
        "mdifftreeview",
        "merge",
        "mmergetreeview",
    ]
    if CONFIG.HAS_VTK:
        mfile_list += ["mparser"]

    for mfile in mfile_list:
        if args.force:
            touch("CGNS/NAV/%s.pyx" % mfile)
        modextlist += [
            Extension(
                "CGNS.NAV.%s" % mfile,
                ["CGNS/NAV/%s.pyx" % mfile],
                include_dirs=CONFIG.NUMPY_PATH_INCLUDES,
                library_dirs=CONFIG.NUMPY_PATH_LIBRARIES,
                libraries=CONFIG.NUMPY_LINK_LIBRARIES,
            )
        ]
    for m in modnamelist:
        modextlist += [
            Extension(
                "CGNS.NAV.%s" % m,
                ["CGNS/NAV/G/%s.pyx" % m],
                include_dirs=CONFIG.NUMPY_PATH_INCLUDES,
                library_dirs=CONFIG.NUMPY_PATH_LIBRARIES,
                libraries=CONFIG.NUMPY_LINK_LIBRARIES,
            )
        ]
        g = ("CGNS/NAV/T/{}.ui".format(m), "CGNS/NAV/G/{}.pyx".format(m))
        if ("true" not in [cui, crc]) and hasToGenerate(g[0], g[1], args.force):
            modgenlist += [m]

    for m in modgenlist:
        log("Generate from updated Qt templates  ({}): {}".format(cui, m))
        com = "{} --from-imports -o CGNS/NAV/G/{}.pyx CGNS/NAV/T/{}.ui".format(
            cui, m, m
        )
        os.system(fix_path(com))
        com = "{} -X language_level=3 -a CGNS/NAV/G/{}.pyx".format(ccy, m)
        os.system(fix_path(com))

    if hasToGenerate("CGNS/NAV/R/Res.qrc", "CGNS/NAV/Res_rc.py", args.force):
        log("Generate from updated Qt Ressources ({}): Res_rc.py".format(crc))
        com = "{} -o CGNS/NAV/Res_rc.py CGNS/NAV/R/Res.qrc".format(crc)
        os.system(fix_path(com))

    ALL_PACKAGES += ["CGNS.NAV", "CGNS.NAV.test"]
    ALL_EXTENSIONS += modextlist

    if CONFIG.HAS_VTK:
        module_logs.append("NAV   add  build (with VTK)")
    else:
        module_logs.append("NAV   add  build (without VTK)")

else:
    module_logs.append("NAV   skip build *")

installConfigFiles(CONFIG.PRODUCTION_DIR)


for e in ALL_EXTENSIONS:
    e.cython_directives = {"language_level": "3"}

#  -------------------------------------------------------------------------
if CONFIG.HAS_CYTHON:
    from Cython.Distutils import build_ext

    cmd = {"clean": clean, "build_ext": build_ext}
else:
    cmd = {"clean": clean}

for module_log in module_logs:
    log(module_log)
line()

# --- Distutils metadata --------------------------------------------

cls_txt = """
Development Status :: 3 - Alpha
Intended Audience :: Developers
Intended Audience :: Information Technology
Intended Audience :: Science/Research
License :: OSI Approved :: GNU Lesser General Public License v2 (LGPLv2)
Programming Language :: Cython
Programming Language :: Python
Programming Language :: Python :: 3
Programming Language :: Python :: 3.6
Programming Language :: Python :: 3.8
Programming Language :: Python :: 3.9
Programming Language :: Python :: Implementation :: CPython
Topic :: Scientific/Engineering
Topic :: Database
Topic :: Software Development :: Libraries :: Python Modules
Operating System :: Unix
Operating System :: POSIX :: Linux
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows
"""

# -------------------------------------------------------------------------
long_description = (Path(__file__).parent / "README.md").read_text()

# -------------------------------------------------------------------------
setup(
    name=CONFIG.NAME,
    version="{}.{}.{}".format(MAJOR_VERSION, MINOR_VERSION, CONFIG.REVISION),
    description=CONFIG.DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[x for x in cls_txt.split("\n") if x],
    author=CONFIG.AUTHOR,
    author_email=CONFIG.EMAIL,
    license=CONFIG.LICENSE,
    url=CONFIG.URL,
    packages=ALL_PACKAGES,
    scripts=ALL_SCRIPTS,
    ext_modules=ALL_EXTENSIONS,
    cmdclass=cmd,
    install_requires=RUN_REQUIRES,
    setup_requires=SETUP_REQUIRES,
)
# -------------------------------------------------------------------------
# --- last line



# -------------------------------------------------------------------------
# pyCGNS - Python package for CFD General Notation System -
# See license.txt file in the root directory of this Python module source  
# -------------------------------------------------------------------------
#
# this script doesn't need to be used for every doc update, run it once
# using :
#   python lib/gendoc.py > doc/mods/PAT/cgnstypes.txt
#
#
import CGNS.PAT.cgnstypes

nzero = [CGNS.PAT.cgnstypes.C_11, CGNS.PAT.cgnstypes.C_1N, CGNS.PAT.cgnstypes.C_NN]


def gentype2(t):
    s = "\n-----\n\n.. _X%s:\n\n%s\n%s\n" % (t.type, t.type, len(t.type) * '-')
    for nt in t.names:
        if (nt == '{UserDefined}'):
            s += "\n :Name: %s " % nt
        else:
            s += "\n :Name: **%s** " % nt
    for c in t.parents:
        s += "\n :Parent: :ref:`%s <X%s>` " % (c, c)
    dtf = False
    for dt in t.datatype:
        if dt != 'LK':
            if not dtf:
                s += "\n :DataType: "
                dtf = True
            s += " " + dt
    if t.enumerate:
        s += "\n :Enumerate: "
    for c in t.children:
        for cn in c[1]:
            cc = t.cardinality(c[0])
            if ((cn != '{UserDefined}') and (t.cardinality(c[0]) not in nzero)):
                s += "\n :Child: **%s** :ref:`%s <X%s>` (%s)" % (cn, c[0], c[0], cc)
            else:
                s += "\n :Child: %s :ref:`%s <X%s>` (%s)" % (cn, c[0], c[0], cc)
    s += "\n"
    return s


def gentype(t):
    s = "\n-----\n\n.. _X%s:\n\n%s\n%s\n" % (t.type, t.type, len(t.type) * '-')
    s += "\n * Name: "
    for nt in t.names:
        s += "\n\n   - %s " % nt
    s += "\n\n * Data-type: "
    for dt in t.datatype:
        if dt != 'LK':
            s += " " + dt
    if t.enumerate:
        s += "\n * Enumerate: "
    s += "\n * Cardinality: %s" % t.cardinality
    s += "\n * Children\n"
    for c in t.children:
        s += "\n   - :ref:`%s <X%s>` (%s)" % (c[0], c[0], c[1])
    s += "\n\n * Parents\n"
    for c in t.parents:
        s += "\n   - :ref:`%s <X%s>` " % (c, c)
    s += "\n"
    return s


def gentypes():
    s = ""
    ct = CGNS.PAT.cgnstypes.types.keys()
    ct.sort()
    for c in ct:
        s += gentype2(CGNS.PAT.cgnstypes.types[c])
    return s


print(gentypes())

# --- last line



#  -------------------------------------------------------------------------
#  pyCGNS.NAV - Python package for CFD General Notation System - VALidater
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#

import numpy

import CGNS.PAT.cgnskeywords
import CGNS.PAT.cgnsutils


def getChildren(t):
    s = set([c[0] for c in t[2]])
    cl = {}
    for c in t[2]:
        cl[c[0]] = c
    return s, cl


def compareValues(na, nb):
    va = na[1]
    vb = nb[1]
    eps = 1e-12
    if va is None and vb is not None:
        return 1
    if va is not None and vb is None:
        return 2
    if va is None and vb is None:
        return 0
    if (not isinstance(va, numpy.ndarray) or
            not isinstance(vb, numpy.ndarray)):
        return 0
    if va.dtype.char != vb.dtype.char:
        return 3
    if va.size != vb.size:
        return 4
    if va.shape != vb.shape:
        return 5
    if va.dtype.char not in ['c', 'S']:
        vd = numpy.array(numpy.abs(va - vb))
        if numpy.any(vd > eps):
            return 6
    elif va.ravel().tostring() != vb.ravel().tostring():
        return 7
    return 0


def diffAB(ta, tb, path, tag, diag, trace=False):
    diag[path] = []
    if ta[3] != tb[3]:
        diag[path].append(('CT',))
        if trace:
            print('CT %s %s' % (tag, path))
    dta = CGNS.PAT.cgnsutils.getValueDataType(ta)
    dtb = CGNS.PAT.cgnsutils.getValueDataType(tb)
    if dta is not CGNS.PAT.cgnskeywords.MT:
        dnum = compareValues(ta, tb)
        if dnum:
            diag[path].append(('C%d' % dnum,))
            if trace:
                print('C%d %s %s' % (dnum, tag, path))
    (sa, da) = getChildren(ta)
    (sb, db) = getChildren(tb)
    a2b = sa.difference(sb)
    sn = sa.union(sb)
    for cn in sn:
        np = path + '/' + cn
        a = cn in sa
        b = cn in sb
        if not a and b:
            diag[path].append(('NA', np))
            if trace:
                print('NA %s %s' % (tag, np))
        if a and not b:
            diag[path].append(('ND', np))
            if trace:
                print('ND %s %s' % (tag, np))
        if a and b:
            diffAB(da[cn], db[cn], np, tag, diag, trace)
    if not diag[path]:
        del diag[path]



#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source
#  ---------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as C
import CGNS.PAT.cgnserrors as E
import CGNS.PAT.cgnskeywords as K
import numpy as N

data = ["UserDefinedData", None, [], "UserDefinedData_t"]
status = ""
comment = ""
pattern = [data, status, comment]



#!/usr/bin/env python
#  -------------------------------------------------------------------------
#  pyCGNS.APP - Python package for CFD General Notation System - APPlicater
#  See license.txt file in the root directory of this Python module source
#  -------------------------------------------------------------------------
#
import CGNS.MAP
import time

print("CGNS.MAP.load(links)")
flags = CGNS.MAP.S2P_DEFAULTS | CGNS.MAP.S2P_FOLLOWLINKS | CGNS.MAP.S2P_TRACE
start = time.clock()
(tree, links, paths) = CGNS.MAP.load("T1.cgns", flags=flags)
print(links)
end = time.clock()
print("time =", end - start)



#  ---------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source
#  ---------------------------------------------------------------------------
#
import CGNS.PAT.cgnslib as C
import CGNS.PAT.cgnserrors as E
import CGNS.PAT.cgnskeywords as K
import numpy as N

#
data = C.newDimensionalExponents(None)
C.newRind(data, N.array([0, 0, 0, 0, 1, 1]))
C.newDataClass(data)
C.newDimensionalUnits(data)
C.newUserDefinedData(data, "{UserDefinedData}")
C.newDescriptor(data, "{Descriptor}")
#
status = "4.4"
comment = "Full SIDS with all optionals"
pattern = [data, status, comment]
#



#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System - 
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#

from qtpy.QtCore import Qt
from qtpy.QtWidgets import QTableWidget

RAISEVIEW = '@@RAISEVIEW@@'
CLOSEVIEW = '@@CLOSEVIEW@@'
INFO = '@@INFOVIEW@@'

KEYMAPPING = {
    RAISEVIEW: Qt.Key_Space,
    CLOSEVIEW: Qt.Key_Delete,
    INFO: Qt.Key_Enter,
}


# -----------------------------------------------------------------
class Q7ControlTableWidget(QTableWidget):
    def mousePressEvent(self, event):
        self.lastPos = event.globalPos()
        self.lastButton = event.button()
        QTableWidget.mousePressEvent(self, event)

    def keyPressEvent(self, event):
        # kmod = event.modifiers()
        kval = event.key()
        if kval == KEYMAPPING[RAISEVIEW]:
            self.control.raiseView()
            return
        if kval == KEYMAPPING[CLOSEVIEW]:
            self.control.closeView()
            return
        if kval == KEYMAPPING[INFO]:
            self.control.info()
            return
        QTableWidget.keyPressEvent(self, event)

# -----------------------------------------------------------------
